@@ -1,17 +1,17 @@
 <?php
 /**
- * $Horde: horde/admin/cmdshell.php,v 1.9.10.6 2006/02/24 22:48:29 chuck Exp $
+ * $Horde: horde/admin/cmdshell.php,v 1.9.10.10 2009/01/06 15:22:10 jan Exp $
  *
- * Copyright 1999-2006 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ *
+ * @author Chuck Hagenbuch <chuck@horde.org>
  */
 
 @define('HORDE_BASE', dirname(__FILE__) . '/..');
 require_once HORDE_BASE . '/lib/base.php';
-require_once 'Horde/Menu.php';
-require_once 'Horde/Help.php';
 
 if (!Auth::isAdmin()) {
     Horde::fatal('Forbidden.', __FILE__, __LINE__);
@@ -19,7 +19,7 @@ if (!Auth::isAdmin()) {
 
 $title = _("Command Shell");
 require HORDE_TEMPLATES . '/common-header.inc';
-require HORDE_TEMPLATES . '/admin/common-header.inc';
+require HORDE_TEMPLATES . '/admin/menu.inc';
 
 echo '<div style="padding:10px">';
 if ($command = trim(Util::getFormData('cmd'))) {
@@ -45,7 +45,8 @@ if ($command = trim(Util::getFormData('cmd'))) {
 
 <form action="<?php echo $_SERVER['PHP_SELF'] ?>" method="post">
 <?php Util::pformInput() ?>
-<textarea class="fixed" name="cmd" rows="10" cols="60">
+<label for="cmd" class="hidden"><?php echo _("Command") ?></label>
+<textarea class="fixed" id="cmd" name="cmd" rows="10" cols="60">
 <?php if (!empty($command)) echo htmlspecialchars($command) ?></textarea>
 <br />
 <input type="submit" class="button" value="<?php echo _("Execute") ?>" />
