@@ -2,12 +2,12 @@
 /**
  * @package Horde_Framework
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
- * $Horde: framework/Horde/Horde.php,v 1.489.2.109 2008/12/08 02:38:14 chuck Exp $
+ * $Horde: framework/Horde/Horde.php,v 1.489.2.116 2009/08/20 21:43:00 jan Exp $
  */
 
 /** Log */
@@ -77,14 +77,19 @@ class Horde {
         $app = isset($GLOBALS['registry']) ? $GLOBALS['registry']->getApp() : 'horde';
         $message = '[' . $app . '] ' . $message . ' [pid ' . getmypid() . ' on line ' . $line . ' of "' . $file . '"]';
 
-        /* Make sure to log in the system's locale. */
+        /* Make sure to log in the system's locale and timezone. */
         $locale = setlocale(LC_TIME, 0);
         setlocale(LC_TIME, 'C');
+        $tz = getenv('TZ');
+        @putenv('TZ');
 
         $logger->log($message, $priority);
 
-        /* Restore original locale. */
+        /* Restore original locale and timezone. */
         setlocale(LC_TIME, $locale);
+        if ($tz) {
+            @putenv('TZ=' . $tz);
+        }
 
         return true;
     }
@@ -649,6 +654,26 @@ HTML;
         return $vfs;
     }
 
+    /**
+     * Return the driver and parameters for the current mailer configuration.
+     *
+     * @since Horde 3.3.5
+     *
+     * @return array  Array with driver name and parameter hash.
+     */
+    function getMailerConfig()
+    {
+        $mail_driver = $GLOBALS['conf']['mailer']['type'];
+        $mail_params = $GLOBALS['conf']['mailer']['params'];
+        if ($mail_driver == 'smtp' && $mail_params['auth'] &&
+            empty($mail_params['username'])) {
+            $mail_params['username'] = Auth::getAuth();
+            $mail_params['password'] = Auth::getCredential('password');
+        }
+
+        return array($mail_driver, $mail_params);
+    }
+
     /**
      * Checks if all necessary parameters for a driver configuration
      * are set and throws a fatal error with a detailed explanation
@@ -839,7 +864,7 @@ HTML;
      */
     function externalUrl($url, $tag = false)
     {
-        if (isset($_COOKIE[session_name()]) ||
+        if (!isset($_GET[session_name()]) ||
             String::substr($url, 0, 1) == '#' ||
             String::substr($url, 0, 7) == 'mailto:') {
             $ext = $url;
@@ -1352,7 +1377,7 @@ HTML;
         $rtl = isset($GLOBALS['nls']['rtl'][$GLOBALS['language']]);
 
         if (!is_array($apps)) {
-            $apps = array($apps);
+            $apps = is_null($apps) ? array() : array($apps);
         }
         if ($inherit) {
             $key = array_search('horde', $apps);
@@ -1862,7 +1887,7 @@ class Horde_Script_Files {
             } else {
                 $url = Horde::url($registry->get('jsuri', $app) . '/' . $file,
                                   $full, -1);
-                $path .= '/js/';
+                $path = $registry->get('jsfs', $app) . '/'; 
             }
         } else {
             $path = $registry->get('templates', $app) . '/javascript/';
