@@ -18,6 +18,7 @@ define('AUTH_REASON_MSG_PARAM', 'logout_msg');
  *   'badlogin'   --  Bad username and/or password
  *   'browser'    --  A browser change was detected
  *   'failed'     --  Login failed
+ *   'expired'    --  Password has expired
  *   'logout'     --  Logout due to user request
  *   'message'    --  Logout with custom message in AUTH_REASON_MSG_PARAM
  *   'session'    --  Logout due to session expiration
@@ -65,9 +66,9 @@ define('AUTH_REASON_SESSIONIP', 'sessionip');
  * The Auth:: class provides a common abstracted interface into the various
  * backends for the Horde authentication system.
  *
- * $Horde: framework/Auth/Auth.php,v 1.142.10.33 2008/09/22 21:51:56 jan Exp $
+ * $Horde: framework/Auth/Auth.php,v 1.142.10.36 2009/06/10 16:43:29 jan Exp $
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://opensource.org/licenses/lgpl-license.php.
@@ -732,12 +733,23 @@ class Auth {
             }
         }
 
-        // Set the user's last_login information.
+        /* Set the user's last_login information. */
         $host = empty($_SERVER['HTTP_X_FORWARDED_FOR'])
             ? $_SERVER['REMOTE_ADDR']
             : $_SERVER['HTTP_X_FORWARDED_FOR'];
+
+	if ((@include_once 'Net/DNS.php')) {
+	    $resolver = new Net_DNS_Resolver();
+	    $resolver->retry = isset($GLOBALS['conf']['dns']['retry']) ? $GLOBALS['conf']['dns']['retry'] : 1;
+	    $resolver->retrans = isset($GLOBALS['conf']['dns']['retrans']) ? $GLOBALS['conf']['dns']['retrans'] : 1;
+	    $response = $resolver->query($host, 'PTR');
+	    $ptrdname = $response ? $response->answer[0]->ptrdname : $host;
+	} else {
+	    $ptrdname = @gethostbyaddr($host);
+	}
+
         $last_login = array('time' => time(),
-                            'host' => @gethostbyaddr($host));
+                            'host' => $ptrdname);
         $GLOBALS['prefs']->setValue('last_login', serialize($last_login));
 
         if ($changeRequested) {
