@@ -6,9 +6,9 @@ require_once 'Horde/Crypt.php';
  * Horde_Crypt_smime:: provides a framework for Horde applications to
  * interact with the OpenSSL library and implement S/MIME.
  *
- * $Horde: framework/Crypt/Crypt/smime.php,v 1.49.2.20 2008/12/07 19:36:36 slusarz Exp $
+ * $Horde: framework/Crypt/Crypt/smime.php,v 1.49.2.22 2009/01/22 10:49:48 jan Exp $
  *
- * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -247,7 +247,9 @@ class Horde_Crypt_smime extends Horde_Crypt {
         $output = $this->_createTempFile('horde-smime');
 
         /* Write text to file. */
-        file_put_contents($input, $data);
+        $fp = fopen($input, 'w');
+        fwrite($fp, $data);
+        fclose($fp);
         unset($data);
 
         exec($sslpath . ' smime -verify -noverify -nochain -in ' . $input . ' -out ' . $output);
