@@ -3,9 +3,9 @@
  * The Group_ldap class provides an LDAP backend for the Horde groups
  * system.
  *
- * $Horde: framework/Group/Group/ldap.php,v 1.18.2.5 2007/01/02 13:54:20 jan Exp $
+ * $Horde: framework/Group/Group/ldap.php,v 1.18.2.15 2009/04/04 10:10:54 jan Exp $
  *
- * Copyright 2005-2007 Ben Chavet <ben@horde.org>
+ * Copyright 2005-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -22,7 +22,7 @@ class Group_ldap extends Group {
     var $_ds;
 
     /**
-     * Local copy of the global $conf['group']['params'] array.  Simply
+     * Local copy of the global $conf['group']['params'] array. Simply
      * for coding convenience.
      */
     var $_params;
@@ -35,15 +35,14 @@ class Group_ldap extends Group {
     /**
      * Constructor.
      */
-    function Group_ldap()
+    function Group_ldap($params)
     {
-        global $conf;
-        $this->_params = $conf['group']['params'];
+        $this->_params = $GLOBALS['conf']['group']['params'];
 
-        $this->_params['gid'] = strtolower($this->_params['gid']);
-        $this->_params['memberuid'] = strtolower($this->_params['memberuid']);
+        $this->_params['gid'] = String::lower($this->_params['gid']);
+        $this->_params['memberuid'] = String::lower($this->_params['memberuid']);
         foreach ($this->_params['newgroup_objectclass'] as $key => $val) {
-            $this->_params['newgroup_objectclass'][$key] = strtolower($val);
+            $this->_params['newgroup_objectclass'][$key] = String::lower($val);
         }
 
         /* Generate LDAP search filter. */
@@ -59,7 +58,7 @@ class Group_ldap extends Group {
             }
         }
 
-        $this->_filter = strtolower($this->_filter);
+        $this->_filter = String::lower($this->_filter);
     }
 
     /**
@@ -72,7 +71,7 @@ class Group_ldap extends Group {
         /* Connect to the LDAP server. */
         $this->_ds = @ldap_connect($this->_params['hostspec']);
         if (!$this->_ds) {
-            return PEAR::raiseError(_("Failed to connect to LDAP server."));
+            return PEAR::raiseError(_("Could not reach the LDAP server"));
         }
 
         if (!ldap_set_option($this->_ds, LDAP_OPT_PROTOCOL_VERSION,
@@ -85,6 +84,17 @@ class Group_ldap extends Group {
                         __FILE__, __LINE__));
         }
 
+        /* Start TLS if we're using it. */
+        if (!empty($this->_params['tls'])) {
+            if (!@ldap_start_tls($this->_ds)) {
+                Horde::logMessage(
+                    sprintf('STARTTLS failed: [%d] %s',
+                            @ldap_errno($this->_ds),
+                            @ldap_error($this->_ds)),
+                    __FILE__, __LINE__, PEAR_LOG_ERR);
+            }
+        }
+
         if (isset($this->_params['binddn'])) {
             $bind = @ldap_bind($this->_ds, $this->_params['binddn'],
                                $this->_params['password']);
@@ -93,7 +103,7 @@ class Group_ldap extends Group {
         }
 
         if (!$bind) {
-            return PEAR::raiseError(_("Could not bind to LDAP server."));
+            return PEAR::raiseError(_("Could not bind to LDAP server"));
         }
 
         return true;
@@ -108,20 +118,20 @@ class Group_ldap extends Group {
     {
         $search = @ldap_list($this->_ds, $dn, 'objectclass=*', array(''));
         if (!$search) {
-            return PEAR::raiseError(_("Could not reach the LDAP server."));
+            return PEAR::raiseError(_("Could not reach the LDAP server"));
         }
 
         $children = @ldap_get_entries($this->_ds, $search);
         for ($i = 0; $i < $children['count']; $i++) {
             $result = $this->_recursive_delete($children[$i]['dn']);
             if (!$result) {
-                return PEAR::raiseError(sprintf(_("Group_ldap: Unable to delete group %s"), $this->getName($children[$i]['dn'])));
+                return PEAR::raiseError(sprintf(_("Group_ldap: Unable to delete group \"%s\". This is what the server said: %s"), $this->getName($children[$i]['dn']), @ldap_error($this->_ds)));
             }
         }
 
         $result = @ldap_delete($this->_ds, $dn);
         if (!$result) {
-            return PEAR::raiseError(sprintf(_("Group_ldap: Unable to delete group %s"), $dn));
+            return PEAR::raiseError(sprintf(_("Group_ldap: Unable to delete group \"%s\". This is what the server said: %s"), $dn, @ldap_error($this->_ds)));
         }
 
         return $result;
@@ -141,10 +151,9 @@ class Group_ldap extends Group {
 
         $search = @ldap_search($this->_ds, $this->_params['basedn'], $this->_filter);
         if (!$search) {
-            return PEAR::raiseError(_("Could not reach the LDAP server."));
+            return PEAR::raiseError(_("Could not reach the LDAP server"));
         }
 
-        @ldap_sort($this->_ds, $search, 'gidnumber');
         $result = @ldap_get_entries($this->_ds, $search);
         @ldap_close($this->_ds);
 
@@ -152,14 +161,21 @@ class Group_ldap extends Group {
             return 1;
         }
 
-        return $result[$result['count'] - 1]['gidnumber'][0] + 1;
+        $nextgid = 0;
+        for ($i = 0; $i < $result['count']; $i++) {
+            if ($result[$i]['gidnumber'][0] > $nextgid) {
+                $nextgid = $result[$i]['gidnumber'][0];
+            }
+        }
+
+        return $nextgid + 1;
     }
 
     /**
      * Return a new group object.
      *
      * @param string $name    The group's name.
-     * @param string $parent  The group's parent's name.
+     * @param string $parent  The group's parent's ID (DN)
      *
      * @return LDAP_Group  A new group object.
      */
@@ -169,12 +185,8 @@ class Group_ldap extends Group {
             return PEAR::raiseError(_("Group names must be non-empty"));
         }
 
-        global $conf;
-        if (!empty($conf['hooks']['groupldap'])) {
-            @include HORDE_BASE . '/config/hooks.php';
-            if (function_exists('_horde_hook_groupldap')) {
-                $entry = call_user_func('_horde_hook_groupldap', $name, $parent);
-            }
+        if (!empty($GLOBALS['conf']['hooks']['groupldap'])) {
+            $entry = Horde::callHook('_horde_hook_groupldap', array($name, $parent));
         } else {
             // Try this simple default and hope it works.
             $entry[$this->_params['gid']] = $name;
@@ -182,7 +194,7 @@ class Group_ldap extends Group {
             $entry['gidnumber'] = $this->_nextGid();
         }
 
-        $group = &new LDAP_Group($name, $parent);
+        $group = new LDAP_Group($name, $parent);
         $group->_fromAttributes($entry);
         $group->setGroupOb($this);
         return $group;
@@ -192,9 +204,6 @@ class Group_ldap extends Group {
      * Return an LDAP_Group object corresponding to the named group, with the
      * users and other data retrieved appropriately.
      *
-     * NOTE: getGroupById() should be used where possible instead of
-     * getGroup() because group names are potentially non-unique values.
-     *
      * @param string $name  The name of the group to retrieve.
      *
      * @return LDAP_Group  The requested group.
@@ -219,11 +228,7 @@ class Group_ldap extends Group {
      */
     function &getGroupById($dn)
     {
-        static $cache;
-
-        if (!is_array($cache)) {
-            $cache = array();
-        }
+        static $cache = array();
 
         if (!isset($cache[$dn])) {
             /* Connect to the LDAP server. */
@@ -234,13 +239,13 @@ class Group_ldap extends Group {
 
             $search = @ldap_search($this->_ds, $dn, $this->_filter);
             if (!$search) {
-                return PEAR::raiseError(_("Could not reach the LDAP server."));
+                return PEAR::raiseError(_("Could not reach the LDAP server"));
             }
 
             $result = @ldap_get_entries($this->_ds, $search);
             @ldap_close($this->_ds);
             if (!is_array($result) || (count($result) <= 1)) {
-                return PEAR::raiseError(_("Empty result."));
+                return PEAR::raiseError(_("Empty result"));
             }
 
             $attributes = array();
@@ -261,6 +266,7 @@ class Group_ldap extends Group {
             $group->setGroupOb($this);
             $cache[$dn] = $group;
         }
+
         return $cache[$dn];
     }
 
@@ -304,7 +310,7 @@ class Group_ldap extends Group {
         $success = @ldap_add($this->_ds, $dn, $entry);
 
         if (!$success) {
-            return PEAR::raiseError(sprintf(_("Group_ldap: Unable to add group %s. This is what the server said: "), $group->getName()) . @ldap_error($this->_ds));
+            return PEAR::raiseError(sprintf(_("Group_ldap: Unable to add group \"%s\". This is what the server said: "), $group->getName()) . @ldap_error($this->_ds));
         }
 
         @ldap_close($this->_ds);
@@ -333,9 +339,12 @@ class Group_ldap extends Group {
             return PEAR::raiseError($success->getMessage());
         }
 
+        // Do not attempt to change an LDAP object's objectClasses
+        unset($entry['objectclass']);
+
         $result = @ldap_modify($this->_ds, $group->getId(), $entry);
         if (!$result) {
-            return PEAR::raiseError(sprintf(_("Group_ldap: Unable to update group %s"), $group->getName()));
+            return PEAR::raiseError(sprintf(_("Group_ldap: Unable to update group \"%s\". This is what the server said: %s"), $group->getName(), @ldap_error($this->_ds)));
         }
 
         @ldap_close($this->_ds);
@@ -353,41 +362,6 @@ class Group_ldap extends Group {
         return $result;
     }
 
-    /**
-     * Change the name of a group.
-     *
-     * @param DataTreeObject_Group $group  The group to rename.
-     * @param string $newName              The group's new name.
-     */
-    function renameGroup($group, $newName)
-    {
-        if (!is_a($group, 'DataTreeObject_Group')) {
-            return PEAR::raiseError('Groups must be DataTreeObject_Group objects or extend that class.');
-        }
-
-        $group->set($this->_params['gid'], $newName);
-        $entry = $group->_toAttributes();
-
-        /* Connect to the LDAP server. */
-        $success = $this->_connect();
-        if (is_a($success, 'PEAR_Error')) {
-            return PEAR::raiseError($success->getMessage());
-        }
-
-        $result = @ldap_modify($this->_ds, $group->getId(), $entry);
-        if (!$result) {
-            return PEAR::raiseError(sprintf(_("Group_ldap: Unable to update group %s"), $group->getName()));
-        }
-
-        @ldap_close($this->_ds);
-
-        /* History Log the name change of the group. */
-        $history = &Horde_History::singleton();
-        $history->log($group->get('dn'), array('action' => 'rename'), true);
-        return $result;
-
-    }
-
     /**
      * Remove a group from the groups system permanently.
      *
@@ -415,7 +389,7 @@ class Group_ldap extends Group {
         } else {
             $result = @ldap_delete($this->_ds, $dn);
             if (!$result) {
-                return PEAR::raiseError(sprintf(_("Group_ldap: Unable to delete group %s"), $dn));
+                return PEAR::raiseError(sprintf(_("Group_ldap: Unable to delete group \"%s\". This is what the server said: %s"), $dn, @ldap_error($this->_ds)));
             }
         }
     }
@@ -429,7 +403,12 @@ class Group_ldap extends Group {
      */
     function getGroupName($dn)
     {
-        $result = ldap_explode_dn($dn, 1);
+        $dn = String::convertCharset($dn, NLS::getCharset(), 'UTF-8');
+        $result = @ldap_explode_dn($dn, 1);
+        if ($result === false) {
+            return PEAR::raiseError(_("Invalid group ID passed (bad DN syntax)"));
+        }
+
         return $result[0];
     }
 
@@ -459,29 +438,25 @@ class Group_ldap extends Group {
      */
     function getGroupId($group)
     {
+        static $cache = array();
+
         if (is_a($group, 'LDAP_Group')) {
             return $group->get('dn');
         }
 
-        static $cache;
-
-        if (!is_array($cache)) {
-            $cache = array();
-        }
-
         if (!isset($cache[$group])) {
             $this->_connect();
             $search = @ldap_search($this->_ds, $this->_params['basedn'],
                                    $this->_params['gid'] . '=' . $group,
                                    array($this->_params['gid']));
             if (!$search) {
-                return PEAR::raiseError(_("Could not reach the LDAP server."));
+                return PEAR::raiseError(_("Could not reach the LDAP server"));
             }
 
             $result = @ldap_get_entries($this->_ds, $search);
             @ldap_close($this->_ds);
             if (!is_array($result) || (count($result) <= 1)) {
-                return PEAR::raiseError(_("Empty result."));
+                return PEAR::raiseError(_("Empty result"));
             }
             $cache[$group] = $result[0]['dn'];
         }
@@ -498,11 +473,7 @@ class Group_ldap extends Group {
      */
     function exists($group)
     {
-        static $cache;
-
-        if (!is_array($cache)) {
-            $cache = array();
-        }
+        static $cache = array();
 
         if (!isset($cache[$group])) {
             /* Connect to the LDAP server. */
@@ -511,14 +482,15 @@ class Group_ldap extends Group {
                 return PEAR::raiseError($success->getMessage());
             }
 
-            $search = @ldap_search($this->_ds, $this->_params['basedn'], '(&' . $this->_filter . '(' . $this->_params['gid'] . '=' . $group . '))');
-            if (!$search) {
-                return PEAR::raiseError(_("Could not reach the LDAP server."));
-            }
+            $groupDN = $this->getGroupId($group);
+            $group = $this->getGroupShortName($group);
 
-            $result = @ldap_get_entries($this->_ds, $search);
-            @ldap_close($this->_ds);
-            $cache[$group] = (is_array($result) && (count($result > 1)));
+            $res = @ldap_compare($this->_ds, $groupDN, $this->_params['gid'], $group);
+            if ($res === false) {
+                return PEAR::raiseError(sprintf(_("Internal Error: An attribute must ALWAYS match itself: %s"), @ldap_error($this->_ds)));
+            }
+            // $res is True if the group exists, -1 if not, false never
+            $cache[$group] = ($res === true);
         }
 
         return $cache[$group];
@@ -527,14 +499,19 @@ class Group_ldap extends Group {
     /**
      * Get a list of the parents of a child group.
      *
-     * @param string $dn  The dn of the child group.
+     * @param string $dn  The fully qualified group dn
      *
-     * @return array  The dn and name of the parent group.
+     * @return array  Nested array of parents
      */
     function getGroupParents($dn)
     {
         $parent = $this->getGroupParent($dn);
-        return array($parent => $this->getGroupName($parent));
+        $parents = array(DATATREE_ROOT => 1);
+        while ($parent != DATATREE_ROOT) {
+            $parents = array($parent => $parents);
+            $parent = $this->getGroupParent($parent);
+        }
+        return $parents;
     }
 
     /**
@@ -546,12 +523,16 @@ class Group_ldap extends Group {
      */
     function getGroupParent($dn)
     {
-        $result = ldap_explode_dn($dn, 0);
+        $result = @ldap_explode_dn($dn, 0);
+        if ($result === false) {
+            return PEAR::raiseError(_("Invalid group ID passed (bad DN syntax)"));
+        }
+
         unset($result['count']);
         unset($result[0]);
         $parent_dn = implode(',', $result);
 
-        if ($parent_dn == $GLOBALS['conf']['group']['params']['basedn']) {
+        if (String::lower($parent_dn) == String::lower($GLOBALS['conf']['group']['params']['basedn'])) {
             return DATATREE_ROOT;
         } else {
             return $parent_dn;
@@ -569,9 +550,12 @@ class Group_ldap extends Group {
      */
     function getGroupParentList($dn)
     {
-        $result = ldap_explode_dn($dn, 0);
-        $num = $result['count'];
+        $result = @ldap_explode_dn($dn, 0);
+        if ($result === false) {
+            return PEAR::raiseError(_("Invalid group ID passed (bad DN syntax)"));
+        }
 
+        $num = $result['count'];
         unset($result['count']);
         unset($result[0]);
 
@@ -609,7 +593,7 @@ class Group_ldap extends Group {
 
             $search = @ldap_search($this->_ds, $this->_params['basedn'], $this->_filter, array($this->_params['gid']));
             if (!$search) {
-                return PEAR::raiseError(_("Could not reach the LDAP server."));
+                return PEAR::raiseError(_("Could not reach the LDAP server"));
             }
 
             @ldap_sort($this->_ds, $search, $this->_params['gid']);
@@ -622,7 +606,7 @@ class Group_ldap extends Group {
 
             $groups = array();
             for ($i = 0; $i < $result['count']; $i++) {
-                $groups[$result[$i]['dn']] = $result[$i][$this->_params['gid']][0];
+                $groups[$result[$i]['dn']] = $this->getGroupName($result[$i]['dn']);
             }
         }
 
@@ -639,11 +623,7 @@ class Group_ldap extends Group {
      */
     function listAllUsers($dn)
     {
-        static $cache;
-
-        if (!is_array($cache)) {
-            $cache = array();
-        }
+        static $cache = array();
 
         if (!isset($cache[$dn])) {
             $success = $this->_connect();
@@ -684,7 +664,7 @@ class Group_ldap extends Group {
      */
     function getGroupMemberships($user, $parentGroups = false)
     {
-        static $cache;
+        static $cache = array();
 
         if (empty($cache[$user])) {
             /* Connect to the LDAP server. */
@@ -693,20 +673,34 @@ class Group_ldap extends Group {
                 return PEAR::raiseError($success->getMessage());
             }
 
-            $search = @ldap_search($this->_ds, $this->_params['basedn'], '(' . $this->_params['memberuid'] . '=' . $user . ')');
+            // Set up search filter
+            $filter = '(' . $this->_params['memberuid'] . '=';
+            if ($GLOBALS['conf']['group']['params']['attrisdn']) {
+                $filter .= $GLOBALS['conf']['auth']['params']['uid'] . '=';
+            }
+            $filter .= $user;
+            if ($GLOBALS['conf']['group']['params']['attrisdn']) {
+                $filter .= ',' . $GLOBALS['conf']['auth']['params']['basedn'];
+            }
+            $filter .= ')';
+
+            // Perform search
+            $search = @ldap_search($this->_ds, $this->_params['basedn'], $filter);
             if (!$search) {
-                return PEAR::raiseError(_("Could not reach the LDAP server."));
+                return PEAR::raiseError(_("Could not reach the LDAP server"));
             }
 
             $result = @ldap_get_entries($this->_ds, $search);
             @ldap_close($this->_ds);
             if (!is_array($result) || (count($result) <= 1)) {
-                return PEAR::raiseError(_("Empty result."));
+                return array();
             }
 
             $groups = array();
+            $current_charset = NLS::getCharset();
             for ($i = 0; $i < $result['count']; $i++) {
-                $groups[$result[$i]['dn']] = $result[$i][$this->_params['gid']][0];
+                $utf8_dn = String::convertCharset($result[$i]['dn'], 'UTF-8', $current_charset);
+                $groups[$utf8_dn] = $this->getGroupName($utf8_dn);
             }
 
             $cache[$user] = $groups;
@@ -721,12 +715,20 @@ class Group_ldap extends Group {
      *
      * @param string $dn  The dn of the object.
      *
-     * @return intenger  The tre depth of the group.
+     * @return intenger  The tree depth of the group.
      */
     function getLevel($dn)
     {
         $base = @ldap_explode_dn($this->_params['basedn'], 0);
+        if ($base === false) {
+            return PEAR::raiseError(_("Invalid basedn configured"));
+        }
+
         $group = @ldap_explode_dn($dn, 0);
+        if ($group === false) {
+            return PEAR::raiseError(_("Invalid group ID passed (bad DN syntax)"));
+        }
+
         return $group['count'] - $base['count'] - 1;
     }
 
@@ -752,10 +754,10 @@ class LDAP_Group extends DataTreeObject_Group {
     {
         parent::DataTreeObject_Group($name);
         if ($parent) {
-            $this->data['dn'] = strtolower($GLOBALS['conf']['group']['params']['gid']) . '=' . $name . ',' . $parent;
+            $this->data['dn'] = String::lower($GLOBALS['conf']['group']['params']['gid']) . '=' . $name . ',' . $parent;
         } else {
-            $this->data['dn'] = strtolower($GLOBALS['conf']['group']['params']['gid']) . '=' . $name .
-                ',' . strtolower($GLOBALS['conf']['group']['params']['basedn']);
+            $this->data['dn'] = String::lower($GLOBALS['conf']['group']['params']['gid']) . '=' . $name .
+                ',' . String::lower($GLOBALS['conf']['group']['params']['basedn']);
         }
     }
 
@@ -791,12 +793,28 @@ class LDAP_Group extends DataTreeObject_Group {
     {
         $this->data['users'] = array();
         foreach ($attributes as $key => $value) {
-            if ($key == strtolower($GLOBALS['conf']['group']['params']['memberuid'])) {
+            if (String::lower($key) == String::lower($GLOBALS['conf']['group']['params']['memberuid'])) {
                 if (is_array($value)) {
                     foreach ($value as $user) {
+                        if ($GLOBALS['conf']['group']['params']['attrisdn']) {
+                            $pattern = '/^' . $GLOBALS['conf']['auth']['params']['uid'] . '=([^,]+).*$/';
+                            $results = array();
+                            preg_match($pattern, $user, $results);
+                            if (isset($results[1])) {
+                                $user = $results[1];
+                            }
+                        }
                         $this->data['users'][$user] = '1';
                     }
                 } else {
+                    if ($GLOBALS['conf']['group']['params']['attrisdn']) {
+                        $pattern = '/^' . $GLOBALS['conf']['auth']['params']['uid'] . '=([^,]+).*$/';
+                        $results = array();
+                        preg_match($pattern, $value, $results);
+                        if (isset($results[1])) {
+                            $value = $results[1];
+                        }
+                    }
                     $this->data['users'][$value] = '1';
                 }
             } elseif ($key == 'mail') {
@@ -819,11 +837,17 @@ class LDAP_Group extends DataTreeObject_Group {
         foreach ($this->data as $key => $value) {
             if ($key == 'users') {
                 foreach ($value as $user => $membership) {
-                    $attributes[strtolower($GLOBALS['conf']['group']['params']['memberuid'])][] = $user;
+                    if ($GLOBALS['conf']['group']['params']['attrisdn']) {
+                        $user = $GLOBALS['conf']['auth']['params']['uid'] .
+                            '=' . $user . ',' . $GLOBALS['conf']['auth']['params']['basedn'];
+                    }
+                    $attributes[String::lower($GLOBALS['conf']['group']['params']['memberuid'])][] = $user;
                 }
             } elseif ($key == 'email') {
-                $attributes['mail'] = !empty($value) ? $value : ' ';
-            } elseif ($key != 'dn' && $key != strtolower($GLOBALS['conf']['group']['params']['memberuid'])) {
+                if (!empty($value)) {
+                    $attributes['mail'] = $value;
+                }
+            } elseif ($key != 'dn' && $key != String::lower($GLOBALS['conf']['group']['params']['memberuid'])) {
                 $attributes[$key] = !empty($value) ? $value : ' ';
             }
         }
@@ -831,4 +855,4 @@ class LDAP_Group extends DataTreeObject_Group {
         return $attributes;
     }
 
-}
\ No newline at end of file
+}
