@@ -2,9 +2,9 @@
 /**
  * The Group:: class provides the Horde groups system.
  *
- * $Horde: framework/Group/Group/sql.php,v 1.1.2.9 2008/11/17 19:12:07 mrubinsk Exp $
+ * $Horde: framework/Group/Group/sql.php,v 1.1.2.12 2009/02/25 05:35:42 chuck Exp $
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -718,7 +718,8 @@ class Group_sql extends Group {
         /* Connect to the sql server using the supplied parameters. */
         require_once 'DB.php';
         $this->_write_db = DB::connect($this->_params,
-                                       array('persistent' => !empty($this->_params['persistent'])));
+                                       array('persistent' => !empty($this->_params['persistent']),
+                                             'ssl' => !empty($this->_params['ssl'])));
         if (is_a($this->_write_db, 'PEAR_Error')) {
             Horde::fatal($this->_write_db, __FILE__, __LINE__);
         }
@@ -736,7 +737,8 @@ class Group_sql extends Group {
         if (!empty($this->_params['splitread'])) {
             $params = array_merge($this->_params, $this->_params['read']);
             $this->_db = DB::connect($params,
-                                     array('persistent' => !empty($params['persistent'])));
+                                     array('persistent' => !empty($params['persistent']),
+                                           'ssl' => !empty($params['ssl'])));
             if (is_a($this->_db, 'PEAR_Error')) {
                 Horde::fatal($this->_db, __FILE__, __LINE__);
             }
