@@ -2,9 +2,9 @@
 /**
  * IMAP_Thread provides functions for working with imap_thread() output.
  *
- * $Horde: framework/IMAP/IMAP/Thread.php,v 1.4.10.16 2007/02/16 07:00:58 slusarz Exp $
+ * $Horde: framework/IMAP/IMAP/Thread.php,v 1.4.10.19 2009/01/06 15:23:11 jan Exp $
  *
- * Copyright 2004-2007 Michael Slusarz <slusarz@curecanti.org>
+ * Copyright 2004-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (GPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/gpl.html.
@@ -15,13 +15,6 @@
  */
 class IMAP_Thread {
 
-    /**
-     * imap_thread() output.
-     *
-     * @var array
-     */
-    var $_threadarray;
-
     /**
      * Internal thread data structure.
      *
@@ -43,8 +36,7 @@ class IMAP_Thread {
      */
     function IMAP_Thread($ob)
     {
-        $this->_threadarray = $ob;
-        $this->_processStructure();
+        $this->_processStructure($ob);
     }
 
    /**
@@ -58,13 +50,9 @@ class IMAP_Thread {
     function getThreadIndent($index)
     {
         $key = $this->_getKey($index);
-        if (!is_null($key)) {
-            if (isset($this->_thread[$key]['level'])) {
-                return $this->_thread[$key]['level'];
-            }
-        }
-
-         return false;
+        return (!is_null($key) && isset($this->_thread[$key]['level']))
+            ? $this->_thread[$key]['level']
+            : false;
     }
 
     /**
@@ -79,13 +67,9 @@ class IMAP_Thread {
     function getThreadBase($index)
     {
         $key = $this->_getKey($index);
-        if (!is_null($key)) {
-            if (!empty($this->_thread[$key]['base'])) {
-                return $this->_thread[$key]['base'];
-            }
-        }
-
-        return false;
+        return (!is_null($key) && !empty($this->_thread[$key]['base']))
+            ? $this->_thread[$key]['base']
+            : false;
     }
 
     /**
@@ -100,13 +84,9 @@ class IMAP_Thread {
     function lastInLevel($index)
     {
         $key = $this->_getKey($index);
-        if (!is_null($key)) {
-            if (!empty($this->_thread[$key]['last'])) {
-                return $this->_thread[$key]['last'];
-            }
-        }
-
-        return false;
+        return (!is_null($key) && !empty($this->_thread[$key]['last']))
+            ? $this->_thread[$key]['last']
+            : false;
     }
 
     /**
@@ -173,15 +153,17 @@ class IMAP_Thread {
      * Process the output from imap_thread() into an internal data structure.
      *
      * @access private
+     *
+     * @param array $ob  Output from imap_thread().
      */
-    function _processStructure()
+    function _processStructure($ob)
     {
         $container = $container_base = $last_index = $thread_base = $thread_base_idx = null;
         $indices = array();
         $i = $last_i = $level = 0;
 
-        reset($this->_threadarray);
-        while (list($key, $val) = each($this->_threadarray)) {
+        reset($ob);
+        while (list($key, $val) = each($ob)) {
             $pos = strpos($key, '.');
             $index = substr($key, 0, $pos);
             $type = substr($key, $pos + 1);
@@ -205,7 +187,7 @@ class IMAP_Thread {
 
             case 'next':
                 if (!is_null($container) && ($container === $index)) {
-                    $container_base = $this->_threadarray[$val . '.num'];
+                    $container_base = $ob[$val . '.num'];
                 } else {
                     $this->_thread[$index]['base'] = (!is_null($container))
                         ? $container_base
