@@ -6,9 +6,9 @@ require_once 'Horde/Util.php';
  * This class defines the Horde_Image:: API, and also provides some
  * utility functions, such as generating highlights of a color.
  *
- * $Horde: framework/Image/Image.php,v 1.39.10.17 2008/06/06 04:17:20 chuck Exp $
+ * $Horde: framework/Image/Image.php,v 1.39.10.20 2009/03/23 18:15:47 mrubinsk Exp $
  *
- * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -89,6 +89,13 @@ class Horde_Image {
      */
     var $_loadedEffects = array();
 
+    /**
+     * What kind of images should ImageMagick generate? Defaults to 'png'.
+     *
+     * @var string
+     */
+    var $_type = 'png';
+
     /**
      * Constructor.
      *
@@ -360,12 +367,12 @@ class Horde_Image {
         if ($filename != $this->_id) {
             $this->reset();
             if (!file_exists($filename)) {
-                return PEAR::raiseError('The image file ' . $image . ' does not exist.');
+                return PEAR::raiseError('The image file ' . $filename . ' does not exist.');
             }
             if ($this->_data = file_get_contents($filename)) {
                 $this->_id = $filename;
             } else {
-                return PEAR::raiseError('Could not load the image file ' . $image);
+                return PEAR::raiseError('Could not load the image file ' . $filename);
             }
         }
     }
@@ -551,6 +558,7 @@ class Horde_Image {
             list($app, $driver) = $driver;
         }
 
+        $driver = basename($driver);
         $class = 'Horde_Image_' . $driver;
         if (!class_exists($class)) {
             if (!empty($app)) {
