@@ -7,9 +7,9 @@ require_once dirname(__FILE__) . '/../Image.php';
  * extension. It mainly provides some utility functions, such as the
  * ability to make pixels, for now.
  *
- * $Horde: framework/Image/Image/gd.php,v 1.48.10.19 2008/01/02 11:30:35 jan Exp $
+ * $Horde: framework/Image/Image/gd.php,v 1.48.10.23 2009/04/14 04:13:20 mrubinsk Exp $
  *
- * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -36,13 +36,6 @@ class Horde_Image_gd extends Horde_Image {
                                'canvas',
                          );
 
-    /**
-     * What kind of images should GD generate? Defaults to 'png'.
-     *
-     * @var string
-     */
-    var $_type = 'png';
-
     /**
      * GD Image resource for the current image data.
      *
@@ -1092,8 +1085,9 @@ class Horde_Image_gd extends Horde_Image {
         }
         error_reporting($GLOBALS['conf']['debug_level']);
         if (!empty($php_errormsg)) {
+            $error_msg = $php_errormsg;
             require_once 'PEAR.php';
-            $result = PEAR::raiseError($function . ': ' . $php_errormsg);
+            $result = PEAR::raiseError($function . ': ' . $error_msg);
         }
         return $result;
     }
