@@ -40,9 +40,9 @@ define('MIME_DEFAULT_ENCODING', '7bit');
  * The MIME_Part:: class provides a wrapper around MIME parts and methods
  * for dealing with them.
  *
- * $Horde: framework/MIME/MIME/Part.php,v 1.177.4.25 2008/12/03 06:48:58 slusarz Exp $
+ * $Horde: framework/MIME/MIME/Part.php,v 1.177.4.27 2009/06/17 19:49:13 slusarz Exp $
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -890,7 +890,8 @@ class MIME_Part {
              * parameter.  See:
              *   http://lists.horde.org/archives/dev/Week-of-Mon-20040426/014240.html */
             if (!empty($GLOBALS['conf']['mailformat']['brokenrfc2231']) &&
-                (strpos($encode_2231, '*=') !== false)) {
+                ((strpos($encode_2231, '*=') !== false) ||
+                 (strpos($encode_2231, '*0=') !== false))) {
                 $ctype .= '; ' . $key . '="' . MIME::encode($value, $charset) . '"';
             }
             $ctype .= '; ' . $encode_2231;
@@ -918,7 +919,8 @@ class MIME_Part {
                 $encode_2231 = MIME::encodeRFC2231('filename', $name, $charset);
                 /* Same broken RFC 2231 workaround as above. */
                 if (!empty($GLOBALS['conf']['mailformat']['brokenrfc2231']) &&
-                    (strpos($encode_2231, '*=') !== false)) {
+                    ((strpos($encode_2231, '*=') !== false) ||
+                     (strpos($encode_2231, '*0=') !== false))) {
                     $disp .= '; filename="' . MIME::encode($name, $charset) . '"';
                 }
                 $disp .= '; ' . $encode_2231;
