@@ -1,6 +1,6 @@
 <?php
 /**
- * $Horde: framework/NLS/NLS.php,v 1.82.4.21 2008/08/26 16:17:32 jan Exp $
+ * $Horde: framework/NLS/NLS.php,v 1.82.4.26 2009/08/28 09:01:53 jan Exp $
  *
  * @package Horde_NLS
  */
@@ -13,7 +13,7 @@ require_once 'Horde/String.php';
  * methods for handling language detection and selection, timezones, and
  * hostname->country lookups.
  *
- * Copyright 1999-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -133,6 +133,7 @@ class NLS {
             }
         }
 
+        @putenv('LC_ALL=' . $lang_charset);
         @putenv('LANG=' . $lang_charset);
         @putenv('LANGUAGE=' . $lang_charset);
     }
@@ -500,7 +501,21 @@ class NLS {
 
         $checkHost = $host;
         if (preg_match('/^\d+\.\d+\.\d+\.\d+$/', $host)) {
-            $checkHost = @gethostbyaddr($host);
+            if ((@include_once 'Net/DNS.php')) {
+                $resolver = new Net_DNS_Resolver();
+                $resolver->retry = isset($GLOBALS['conf']['dns']['retry']) ? $GLOBALS['conf']['dns']['retry'] : 1;
+                $resolver->retrans = isset($GLOBALS['conf']['dns']['retrans']) ? $GLOBALS['conf']['dns']['retrans'] : 1;
+                if ($response = $resolver->query($host, 'PTR')) {
+                    foreach ($response->answer as $val) {
+                        if (isset($val->ptrdname)) {
+                            $checkHost = $val->ptrdname;
+                            break;
+                        }
+                    }
+                }
+            } else {
+                $checkHost = @gethostbyaddr($host);
+            }
         }
 
         /* Get the TLD of the hostname. */
