@@ -2,13 +2,12 @@
 /**
  * This is a utility class, every method is static.
  *
- * $Horde: framework/SQL/SQL.php,v 1.30.2.5 2005/03/16 17:23:27 chuck Exp $
+ * $Horde: framework/SQL/SQL.php,v 1.30.2.21 2009/01/06 15:23:33 jan Exp $
  *
- * Copyright 1999-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2005 Jan Schneider <jan@horde.org>
+ * Copyright 1999-2009 The Horde Project (http://www.horde.org/)
  *
- * See the enclosed file COPYING for license information (LGPL).  If you did
- * not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
+ * See the enclosed file COPYING for license information (LGPL). If you
+ * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Jan Schneider <jan@horde.org>
@@ -21,18 +20,19 @@ class Horde_SQL {
      * Returns a boolean expression using the specified operator. Uses
      * database-specific casting, if necessary.
      *
-     * @access public
+     * @param DB $dbh        The PEAR::DB database object.
+     * @param string $lhs    The column or expression to test.
+     * @param string $op     The operator.
+     * @param string $rhs    The comparison value.
+     * @param boolean $bind  If true, the method returns the query and a list
+     *                       of values suitable for binding as an array.
+     * @param array $params  Any additional parameters for the operator. @since
+     *                       Horde 3.2
      *
-     * @param object $dbh  The PEAR::DB database object.
-     * @param string $lhs  The column or expression to test.
-     * @param string $op   The operator.
-     * @param string $rhs  The comparison value.
-     * @param bool $bind   If true, the method returns the query and a list of
-     *                     values suitable for binding as an array.
-     *
-     * @returns string  The SQL test fragment.
+     * @return mixed  The SQL test fragment, or an array containing the query
+     *                and a list of values if $bind is true.
      */
-    function buildClause(&$dbh, $lhs, $op, $rhs, $bind = false)
+    function buildClause(&$dbh, $lhs, $op, $rhs, $bind = false, $params = array())
     {
         switch ($op) {
         case '|':
@@ -54,10 +54,9 @@ class Horde_SQL {
                 }
 
             case 'oci8':
-                // Oracle uses & for variables. We need to use the
-                // bitand function that is available, but may be
-                // unsupported.
-                $query = 'bitand(%s,%d) = %d';
+                // Oracle uses & for variables. We need to use the bitand
+                // function that is available, but may be unsupported.
+                $query = 'bitand(%s, %s) = %s';
                 if ($bind) {
                     return array(sprintf(Horde_SQL::escapePrepare($query),
                                          Horde_SQL::escapePrepare($lhs), '?', '?'),
@@ -66,9 +65,21 @@ class Horde_SQL {
                     return sprintf($query, $lhs, (int)$rhs, (int)$rhs);
                 }
 
+            case 'mssql':
+                // MSSQL must have a valid boolean expression
+                $query = '(CASE WHEN ISNUMERIC(%s) = 1 THEN (%s & %s) ELSE %s END) = %s';
+                if ($bind) {
+                    return array(sprintf(Horde_SQL::escapePrepare($query),
+                                         Horde_SQL::escapePrepare($lhs),
+                                         Horde_SQL::escapePrepare($lhs), '?', '?', '?'),
+                                 array((int)$rhs, (int)$rhs - 1, (int)$rhs));
+                } else {
+                    return sprintf($query, $lhs, $lhs, (int)$rhs, (int)$rhs - 1, (int)$rhs);
+                }
+
             case 'odbc':
                 // ODBC must have a valid boolean expression
-                $query = '(%s & %d) = %d';
+                $query = '(%s & %s) = %s';
                 if ($bind) {
                     return array(sprintf(Horde_SQL::escapePrepare($query),
                                          Horde_SQL::escapePrepare($lhs), '?', '?'),
@@ -98,35 +109,62 @@ class Horde_SQL {
 
         case 'IN':
             if ($bind) {
-                // We need to bind each member of the IN clause
-                // separately to ensure proper quoting.
-                if (substr($rhs, 0, 1) == '(') {
-                    $rhs = substr($rhs, 1);
-                }
-                if (substr($rhs, -1) == ')') {
-                    $rhs = substr($rhs, 0, -1);
-                }
+                if (is_array($rhs)) {
+                    return array($lhs . ' IN (?' . str_repeat(', ?', count($rhs) - 1) . ')', $rhs);
+                } else {
+                    // We need to bind each member of the IN clause
+                    // separately to ensure proper quoting.
+                    if (substr($rhs, 0, 1) == '(') {
+                        $rhs = substr($rhs, 1);
+                    }
+                    if (substr($rhs, -1) == ')') {
+                        $rhs = substr($rhs, 0, -1);
+                    }
 
-                // If we use this with quoted strings instead of
-                // integer ids, we will have additional problems to
-                // solve at some point.
-                $ids = preg_split('/\s*,\s*/', $rhs);
+                    $ids = preg_split('/\s*,\s*/', $rhs);
 
-                return array($lhs . ' IN (?' . str_repeat(', ?', count($ids) - 1) . ')', $ids);
+                    return array($lhs . ' IN (?' . str_repeat(', ?', count($ids) - 1) . ')', $ids);
+                }
             } else {
-                return $lhs . ' IN ' . $rhs;
+                if (is_array($rhs)) {
+                    return $lhs . ' IN ' . implode(', ', $rhs);
+                } else {
+                    return $lhs . ' IN ' . $rhs;
+                }
             }
 
         case 'LIKE':
             if ($dbh->phptype == 'pgsql') {
                 $query = '%s ILIKE %s';
             } else {
-                $query = '%s LIKE LOWER(%s)';
+                $query = 'LOWER(%s) LIKE LOWER(%s)';
             }
             if ($bind) {
-                return array(sprintf($query, Horde_SQL::escapePrepare($lhs), '?'), array('%' . $rhs . '%'));
+                if (empty($params['begin'])) {
+                    return array(sprintf($query,
+                                         Horde_SQL::escapePrepare($lhs),
+                                         '?'),
+                                 array('%' . $rhs . '%'));
+                } else {
+                    return array(sprintf('(' . $query . ' OR ' . $query . ')',
+                                         Horde_SQL::escapePrepare($lhs),
+                                         '?',
+                                         Horde_SQL::escapePrepare($lhs),
+                                         '?'),
+                                 array($rhs . '%', '% ' . $rhs . '%'));
+                }
             } else {
-                return sprintf($query, $lhs, $dbh->quote('%' . $rhs . '%'));
+                if (empty($params['begin'])) {
+                    return sprintf($query,
+                                   $lhs,
+                                   $dbh->quote('%' . $rhs . '%'));
+                } else {
+                    return sprintf('(' . $query . ' OR ' . $query . ')',
+                                   $lhs,
+                                   $dbh->quote($rhs . '%'),
+                                   $lhs,
+                                   $dbh->quote('% ' . $rhs . '%'));
+                }
             }
 
         default:
@@ -263,8 +301,7 @@ class Horde_SQL {
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         /* Execute the query. */
-        $stmt = $this->_db->prepare($query);
-        return $this->_db->execute($stmt, $values);
+        return $this->_db->query($query, $values);
     }
 
     function updateBlob(&$dbh, $table, $field, $data, $where, $alsoupdate)
@@ -335,8 +372,7 @@ class Horde_SQL {
                           __FILE__, __LINE__, PEAR_LOG_DEBUG);
 
         /* Execute the query. */
-        $stmt = $dbh->prepare($query);
-        return $dbh->execute($stmt, $values);
+        return $dbh->query($query, $values);
     }
 
     /**
@@ -353,12 +389,10 @@ class Horde_SQL {
      *
      *      foo = 1, bar = 'hello'
      *
-     * @access public
+     * @param DB $dbh        The PEAR::DB database object.
+     * @param array $values  The array of column => value pairs.
      *
-     * @param object $dbh       The PEAR::DB database object.
-     * @param string $values    The array of column => value pairs.
-     *
-     * @returns string  The SQL SET fragment.
+     * @return string  The SQL SET fragment.
      */
     function updateValues(&$dbh, $values)
     {
@@ -383,12 +417,10 @@ class Horde_SQL {
      *
      *      ( foo, bar ) VALUES ( 1, 'hello' )
      *
-     * @access public
+     * @param DB $dbh        The PEAR::DB database object.
+     * @param array $values  The array of column => value pairs.
      *
-     * @param object $dbh       The PEAR::DB database object.
-     * @param string $values    The array of column => value pairs.
-     *
-     * @returns string  The SQL fragment.
+     * @return string  The SQL fragment.
      */
     function insertValues(&$dbh, $values)
     {
