@@ -1,17 +1,21 @@
 <?php
 /**
- * SessionHandler:: defines an API for implementing custom session
- * handlers for PHP.
+ * SessionHandler:: defines an API for implementing custom PHP session
+ * handlers.
  *
- * $Horde: framework/SessionHandler/SessionHandler.php,v 1.13.10.13 2007/12/05 18:57:35 slusarz Exp $
+ * Optional parameters:<pre>
+ *   'memcache' - (boolean) Use memcache to cache session data?
+ * </pre>
  *
- * Copyright 2002-2007 Mike Cochrane <mike@graftonhall.co.nz>
+ * $Horde: framework/SessionHandler/SessionHandler.php,v 1.13.10.18 2009/01/06 15:23:34 jan Exp $
+ *
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Mike Cochrane <mike@graftonhall.co.nz>
- * @since   Horde 3.0
+ * @author  Michael Slusarz <slusarz@curecanti.org>
  * @package Horde_SessionHandler
  */
 class SessionHandler {
@@ -23,6 +27,27 @@ class SessionHandler {
      */
     var $_params = array();
 
+    /**
+     * Initial session data signature.
+     *
+     * @var string
+     */
+    var $_sig;
+
+    /**
+     * Force saving the session data?
+     *
+     * @var boolean
+     */
+    var $_force = false;
+
+    /**
+     * Has a connection been made to the backend?
+     *
+     * @var boolean
+     */
+    var $_connected = false;
+
     /**
      * Constructs a new SessionHandler object.
      *
@@ -53,26 +78,33 @@ class SessionHandler {
         }
 
         $driver = basename($driver);
+        $persistent_params = array();
+
         if ($driver == 'memcached') {
             // Trap for old driver name.
             $driver = 'memcache';
+        } elseif (($driver != 'memcache') && !empty($params['memcache'])) {
+            unset($params['memcache']);
+            $persistent_params = array('persistent_driver' => $driver, 'persistent_params' => $params);
+            $driver = 'memcache';
+            $params = null;
         }
 
         $class = 'SessionHandler_' . $driver;
         if (!class_exists($class)) {
             if (!empty($app)) {
-                include_once $GLOBALS['registry']->get('fileroot', $app) . '/lib/SessionHandler/' . $driver . '.php';
+                include $GLOBALS['registry']->get('fileroot', $app) . '/lib/SessionHandler/' . $driver . '.php';
             } else {
-                include_once 'Horde/SessionHandler/' . $driver . '.php';
+                include 'Horde/SessionHandler/' . $driver . '.php';
             }
         }
 
         if (class_exists($class)) {
-            if (is_null($params)) {
+            if (empty($params)) {
                 include_once 'Horde.php';
                 $params = Horde::getDriverConfig('sessionhandler', $driver);
             }
-            $handler = new $class($params);
+            $handler = new $class(array_merge($params, $persistent_params));
         } else {
             $handler = PEAR::raiseError('Class definition of ' . $class . ' not found.');
         }
@@ -91,8 +123,8 @@ class SessionHandler {
      * @param string $driver  See SessionHandler::factory().
      * @param array $params   See SessionHandler::factory().
      *
-     * @return mixed  The created concrete SessionHandler instance, or false
-     *                on error.
+     * @return mixed  The created concrete SessionHandler instance, or
+     *                PEAR_Error on error.
      */
     function &singleton($driver, $params = null)
     {
@@ -109,14 +141,40 @@ class SessionHandler {
     /**
      * Open the SessionHandler backend.
      *
-     * @abstract
-     *
      * @param string $save_path     The path to the session object.
      * @param string $session_name  The name of the session.
      *
      * @return boolean  True on success, false otherwise.
      */
     function open($save_path, $session_name)
+    {
+        if ($this->_connected) {
+            return true;
+        }
+
+        $res = $this->_open($save_path, $session_name);
+        if (is_a($res, 'PEAR_Error')) {
+            Horde::logMessage($res, __FILE__, __LINE__, PEAR_LOG_ERR);
+            return false;
+        }
+
+        register_shutdown_function(array(&$this, '_shutdown'));
+        $this->_connected = true;
+        return true;
+    }
+
+    /**
+     * Open the SessionHandler backend.
+     *
+     * @abstract
+     * @access private
+     *
+     * @param string $save_path     The path to the session object.
+     * @param string $session_name  The name of the session.
+     *
+     * @return boolean  True on success, PEAR_Error on error.
+     */
+    function _open($save_path = null, $session_name = null)
     {
         return true;
     }
@@ -124,11 +182,24 @@ class SessionHandler {
     /**
      * Close the SessionHandler backend.
      *
-     * @abstract
-     *
      * @return boolean  True on success, false otherwise.
      */
     function close()
+    {
+        $res = $this->_close();
+        $this->_connected = false;
+        return $res;
+    }
+
+    /**
+     * Close the SessionHandler backend.
+     *
+     * @abstract
+     * @access private
+     *
+     * @return boolean  True on success, false otherwise.
+     */
+    function _close()
     {
         return true;
     }
@@ -136,8 +207,8 @@ class SessionHandler {
     /**
      * Read the data for a particular session identifier from the
      * SessionHandler backend.
-     *
-     * @abstract
+     * This method should only be called internally by PHP via
+     * session_set_save_handler().
      *
      * @param string $id  The session identifier.
      *
@@ -145,13 +216,31 @@ class SessionHandler {
      */
     function read($id)
     {
-        return PEAR::raiseError(_("Not supported."));
+        $result = $this->_read($id);
+        $this->_sig = md5($result);
+        return $result;
+    }
+
+    /**
+     * Read the data for a particular session identifier from the
+     * SessionHandler backend.
+     *
+     * @abstract
+     * @access private
+     *
+     * @param string $id  The session identifier.
+     *
+     * @return string  The session data.
+     */
+    function _read($id)
+    {
+        return '';
     }
 
     /**
      * Write session data to the SessionHandler backend.
-     *
-     * @abstract
+     * This method should only be called internally by PHP via
+     * session_set_save_handler().
      *
      * @param string $id            The session identifier.
      * @param string $session_data  The session data.
@@ -160,12 +249,35 @@ class SessionHandler {
      */
     function write($id, $session_data)
     {
-        return PEAR::raiseError(_("Not supported."));
+        if (!$this->_force && ($this->_sig == md5($session_data))) {
+            Horde::logMessage('Session data unchanged (id = ' . $id . ')', __FILE__, __LINE__, PEAR_LOG_DEBUG);
+            return true;
+        }
+
+        return $this->_write($id, $session_data);
+    }
+
+    /**
+     * Write session data to the SessionHandler backend.
+     *
+     * @abstract
+     * @access private
+     *
+     * @param string $id            The session identifier.
+     * @param string $session_data  The session data.
+     *
+     * @return boolean  True on success, false otherwise.
+     */
+    function _write($id, $session_data)
+    {
+        return false;
     }
 
     /**
      * Destroy the data for a particular session identifier in the
      * SessionHandler backend.
+     * This method should only be called internally by PHP via
+     * session_set_save_handler().
      *
      * @abstract
      *
@@ -175,11 +287,13 @@ class SessionHandler {
      */
     function destroy($id)
     {
-        return PEAR::raiseError(_("Not supported."));
+        return false;
     }
 
     /**
      * Garbage collect stale sessions from the SessionHandler backend.
+     * This method should only be called internally by PHP via
+     * session_set_save_handler().
      *
      * @abstract
      *
@@ -189,40 +303,21 @@ class SessionHandler {
      */
     function gc($maxlifetime = 300)
     {
-        return PEAR::raiseError(_("Not supported."));
+        return false;
     }
 
     /**
-     * Determines if a session belongs to an authenticated user.
+     * Get session data read-only.
      *
      * @access private
      *
-     * @param string $session_data  The session data itself.
-     * @param boolean $return_user  If true, return the user session data.
+     * @param string $id  The session identifier.
      *
-     * @return boolean|string  True or the user's session data if the session
-     *                         belongs to an authenticated user.
+     * @return string  The session data.
      */
-    function _isAuthenticated($session_data, $return_data = false)
+    function _readOnly($id)
     {
-        if (empty($session_data)) {
-            return false;
-        }
-
-        while ($session_data) {
-            $vars = explode('|', $session_data, 2);
-            $data = unserialize($vars[1]);
-            if ($vars[0] == '__auth') {
-                if (empty($data)) {
-                    return false;
-                }
-                if (!empty($data['authenticated'])) {
-                    return $return_data ? $data : true;
-                }
-                return false;
-            }
-            $session_data = substr($session_data, strlen($vars[0]) + strlen(serialize($data)) + 1);
-        }
+        return $this->read($id);
     }
 
     /**
@@ -231,6 +326,7 @@ class SessionHandler {
      * @abstract
      *
      * @return array  A list of valid session identifiers.
+     *                Returns PEAR_Error on error.
      */
     function getSessionIDs()
     {
@@ -239,60 +335,103 @@ class SessionHandler {
 
     /**
      * Determine the number of currently logged in users.
+     * getSessionsInfo() should be called instead.
      *
-     * @return integer  A count of logged in users.
+     * @deprecated
+     *
+     * @return integer  A count of logged in users or PEAR_Error on error.
      */
     function countAuthenticatedUsers()
     {
-        $count = 0;
-
-        $sessions = $this->getSessionIDs();
+        $sessions = $this->getSessionsInfo();
         if (is_a($sessions, 'PEAR_Error')) {
             return $sessions;
         }
-
-        foreach ($sessions as $id) {
-            $data = $this->read($id);
-            if (!is_a($data, 'PEAR_Error') &&
-                $this->_isAuthenticated($data)) {
-                $count++;
-            }
-        }
-
-        return $count;
+        return count($sessions);
     }
 
     /**
      * Returns a list of currently logged in users.
+     * getSessionsInfo() should be called instead.
      *
-     * @return array  A list of logged in users.
+     * @deprecated
+     *
+     * @param boolean $date  Prefix the timestamp to the username?
+     *
+     * @return array  A list of logged in users or PEAR_Error on error.
      */
     function listAuthenticatedUsers($date = false)
     {
-        $users = array();
-
-        $sessions = $this->getSessionIDs();
+        $sessions = $this->getSessionsInfo();
         if (is_a($sessions, 'PEAR_Error')) {
             return $sessions;
         }
 
-        foreach ($sessions as $id) {
-            $data = $this->read($id);
-            if (is_a($data, 'PEAR_Error')) {
-                continue;
-            }
+        $users = array();
 
-            $data = $this->_isAuthenticated($data, true);
-            if ($data !== false && isset($data['userId'])) {
-                $user = $data['userId'];
-                if ($date && isset($data['timestamp'])) {
-                    $user = date('r', $data['timestamp']) . '  ' . $user;
-                }
-                $users[] = $user;
-            }
+        reset($users);
+        while (list($key, $val) = each($users)) {
+            $users[] = (($date) ? date('r', $data['timestamp']) . '  ' : '') . $data['userid'];
         }
 
         return $users;
     }
 
+    /**
+     * Returns a list of authenticated users and data about their session.
+     *
+     * @since Horde 3.2
+     *
+     * @return array  For authenticated users, the sessionid as a key and the
+     *                information returned from Auth::readSessionData() as
+     *                values.
+     *                Returns PEAR_Error on error.
+     */
+    function getSessionsInfo()
+    {
+        $sessions = $this->getSessionIDs();
+        if (is_a($sessions, 'PEAR_Error')) {
+            return $sessions;
+        }
+
+        $info = array();
+
+        foreach ($sessions as $id) {
+            $data = $this->_readOnly($id);
+            if (is_a($data, 'PEAR_Error')) {
+                continue;
+            }
+            $data = Auth::readSessionData($data, true);
+            if ($data !== false) {
+                $info[$id] = $data;
+            }
+        }
+
+        return $info;
+    }
+
+    /**
+     * Shutdown function.  Used to determine if we need to write the session
+     * to avoid a session timeout, even though the session is unchanged.
+     * Theory: On initial login, set the current time plus half of the max
+     * lifetime in the session.  Then check this timestamp before saving.
+     * If we exceed, force a write of the session and set a new timestamp.
+     * Why half the maxlifetime?  It guarantees that if we are accessing the
+     * server via a periodic mechanism (think folder refreshing in IMP) that
+     * we will catch this refresh.
+     *
+     * @access private
+     */
+    function _shutdown()
+    {
+        $curr_time = time();
+
+        if (!isset($_SESSION['__sessionhandler']) ||
+            ($curr_time >= $_SESSION['__sessionhandler'])) {
+
+            $_SESSION['__sessionhandler'] = $curr_time + (ini_get('session.gc_maxlifetime') / 2);
+            $this->_force = true;
+        }
+    }
+
 }
