@@ -4,9 +4,9 @@
  * methods take care of any site-specific restrictions configured in in the
  * application's prefs.php and conf.php files.
  *
- * $Horde: framework/Share/Share.php,v 1.111.2.30 2008/10/24 14:10:43 jan Exp $
+ * $Horde: framework/Share/Share.php,v 1.111.2.33 2009/04/20 21:09:02 jan Exp $
  *
- * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  * Copyright 2002-2007 Infoteck Internet <webmaster@infoteck.qc.ca>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
@@ -109,7 +109,7 @@ class Horde_Share {
             !empty($GLOBALS['conf']['share']['cache'])) {
             require_once 'Horde/SessionObjects.php';
             $session = &Horde_SessionObjects::singleton();
-            $shares[$signature] = &$session->query('horde_share_' . $app . '_' . $driver);
+            $shares[$signature] = &$session->query('horde_share_' . $app . '_' . $driver . '1');
         }
 
         if (empty($shares[$signature])) {
@@ -744,7 +744,10 @@ class Horde_Share_Object {
     function listUsers($perm_level = null)
     {
         $perm = &$this->getPermission();
-        return array_keys($perm->getUserPermissions($perm_level));
+        // Always return the share's owner!!
+        $results = array_keys($perm->getUserPermissions($perm_level));
+        array_push($results, $this->get('owner'));
+        return $results;
     }
 
     /**
