@@ -3,9 +3,9 @@
  * Horde_Share_sql:: provides the sql backend for the horde share
  * driver.
  *
- * $Horde: framework/Share/Share/sql.php,v 1.1.2.52 2008/11/12 19:29:59 mrubinsk Exp $
+ * $Horde: framework/Share/Share/sql.php,v 1.1.2.57 2009/08/12 13:52:00 jan Exp $
  *
- * Copyright 2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2008-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -75,6 +75,20 @@ class Horde_Share_sql extends Horde_Share {
         parent::__wakeup();
     }
 
+    /**
+     * Returns the properties that need to be serialized.
+     *
+     * @return array  List of serializable properties.
+     */
+    function __sleep()
+    {
+        $properties = get_object_vars($this);
+        unset($properties['_sortList'],
+              $properties['_db'],
+              $properties['_write_db']);
+        return array_keys($properties);
+    }
+
     /**
      * Get storage table
      */
@@ -133,7 +147,6 @@ class Horde_Share_sql extends Horde_Share {
             $result->free();
         }
     }
-
     /**
      * Get groups permissions
      *
@@ -646,6 +659,8 @@ class Horde_Share_sql extends Horde_Share {
      */
     function _getShareCriteria($userid, $perm = PERMS_SHOW, $attributes = null)
     {
+        require_once 'Horde/SQL.php';
+
         $query = ' FROM ' . $this->_table . ' s ';
         $where = '';
 
@@ -654,15 +669,15 @@ class Horde_Share_sql extends Horde_Share {
             $where .= 's.share_owner = ' . $this->_write_db->quote($userid);
 
             // (name == perm_creator and val & $perm)
-            $where .= ' OR (s.perm_creator & ' . $perm . ') != 0';
+            $where .= ' OR (' . Horde_SQL::buildClause($this->_db, 's.perm_creator', '&', $perm) . ')';
 
             // (name == perm_creator and val & $perm)
-            $where .= ' OR (s.perm_default & ' . $perm . ') != 0';
+            $where .= ' OR (' . Horde_SQL::buildClause($this->_db, 's.perm_default',  '&', $perm) . ')';
 
             // (name == perm_users and key == $userid and val & $perm)
             $query .= ' LEFT JOIN ' . $this->_table . '_users AS u ON u.share_id = s.share_id';
             $where .= ' OR ( u.user_uid = ' .  $this->_write_db->quote($userid)
-                . ' AND (u.perm & ' . $perm . ') != 0)';
+                . ' AND (' . Horde_SQL::buildClause($this->_db, 'u.perm', '&', $perm) . '))';
 
             // If the user has any group memberships, check for those also.
             require_once 'Horde/Group.php';
@@ -677,12 +692,12 @@ class Horde_Share_sql extends Horde_Share {
                 }
                 $query .= ' LEFT JOIN ' . $this->_table . '_groups AS g ON g.share_id = s.share_id';
                 $where .= ' OR (g.group_uid IN (' . implode(',', $group_ids) . ')'
-                    . ' AND (g.perm & ' . $perm .') != 0)';
+                    . ' AND (' . Horde_SQL::buildClause($this->_db, 'g.perm', '&', $perm) . '))';
             } elseif (is_a($groups, 'PEAR_Error')) {
                 Horde::logMessage($groups, __FILE__, __LINE__, PEAR_LOG_ERR);
             }
         } else {
-            $where = '(s.perm_guest & ' . $perm . ') != 0';
+            $where = '(' . Horde_SQL::buildClause($this->_db, 's.perm_guest', '&', $perm) . ')';
         }
 
         $attributes = $this->_toDriverKeys($attributes);
@@ -736,7 +751,7 @@ class Horde_Share_sql extends Horde_Share {
         require_once 'MDB2.php';
         $params = $this->_params;
         unset($params['charset']);
-        $this->_write_db = MDB2::factory($params);
+        $this->_write_db = &MDB2::factory($params);
         if (is_a($this->_write_db, 'PEAR_Error')) {
             Horde::fatal($this->_write_db, __FILE__, __LINE__);
         }
@@ -752,6 +767,15 @@ class Horde_Share_sql extends Horde_Share {
             $this->_write_db->setOption('field_case', CASE_LOWER);
             $this->_write_db->setOption('portability', MDB2_PORTABILITY_FIX_CASE | MDB2_PORTABILITY_ERRORS | MDB2_PORTABILITY_RTRIM | MDB2_PORTABILITY_FIX_ASSOC_FIELD_NAMES);
             break;
+
+        case 'pgsql':
+            /* The debug handler breaks PostgreSQL. In most cases it shouldn't
+             * be necessary, but this may mean we simply can't support use of
+             * multiple Postgres databases right now. See
+             * http://bugs.horde.org/ticket/7825 */
+            $this->_write_db->setOption('debug', false);
+            // Fall through
+
         default:
             $this->_write_db->setOption('field_case', CASE_LOWER);
             $this->_write_db->setOption('portability', MDB2_PORTABILITY_FIX_CASE | MDB2_PORTABILITY_ERRORS | MDB2_PORTABILITY_FIX_ASSOC_FIELD_NAMES);
@@ -773,6 +797,15 @@ class Horde_Share_sql extends Horde_Share {
                 $this->_db->setOption('field_case', CASE_LOWER);
                 $this->_db->setOption('portability', MDB2_PORTABILITY_FIX_CASE | MDB2_PORTABILITY_ERRORS | MDB2_PORTABILITY_RTRIM | MDB2_PORTABILITY_FIX_ASSOC_FIELD_NAMES);
                 break;
+
+            case 'pgsql':
+                /* The debug handler breaks PostgreSQL. In most cases it shouldn't
+                 * be necessary, but this may mean we simply can't support use of
+                 * multiple Postgres databases right now. See
+                 * http://bugs.horde.org/ticket/7825 */
+                $this->_write_db->setOption('debug', false);
+                // Fall through
+
             default:
                 $this->_db->setOption('field_case', CASE_LOWER);
                 $this->_db->setOption('portability', MDB2_PORTABILITY_FIX_CASE | MDB2_PORTABILITY_ERRORS | MDB2_PORTABILITY_FIX_ASSOC_FIELD_NAMES);
