@@ -6,9 +6,9 @@ $GLOBALS['_HORDE_STRING_CHARSET'] = 'iso-8859-1';
  * The String:: class provides static methods for charset and locale safe
  * string manipulation.
  *
- * $Horde: framework/Util/String.php,v 1.43.6.31 2008/10/23 21:28:38 jan Exp $
+ * $Horde: framework/Util/String.php,v 1.43.6.37 2009/03/30 15:31:38 jan Exp $
  *
- * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2003-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -94,6 +94,7 @@ class String {
 
         if (is_array($input)) {
             $tmp = array();
+            reset($input);
             while (list($key, $val) = each($input)) {
                 $tmp[String::_convertCharset($key, $from, $to)] = String::convertCharset($val, $from, $to);
             }
@@ -317,9 +318,27 @@ class String {
         if (is_null($length)) {
             $length = String::length($string, $charset) - $start;
         }
+
         if ($length == 0) {
             return '';
         }
+
+        /* Try iconv. */
+        if (function_exists('iconv_substr')) {
+            if (is_null($charset)) {
+                $charset = $GLOBALS['_HORDE_STRING_CHARSET'];
+            }
+
+            $old_error = error_reporting(0);
+            $ret = iconv_substr($string, $start, $length, $charset);
+            error_reporting($old_error);
+            /* iconv_substr() returns false on failure. */
+            if ($ret !== false) {
+                return $ret;
+            }
+        }
+
+        /* Try mbstring. */
         if (String::extensionExists('mbstring')) {
             if (is_null($charset)) {
                 $charset = $GLOBALS['_HORDE_STRING_CHARSET'];
@@ -327,10 +346,12 @@ class String {
             $old_error = error_reporting(0);
             $ret = mb_substr($string, $start, $length, String::_mbstringCharset($charset));
             error_reporting($old_error);
-            if (!empty($ret)) {
+            /* mb_substr() returns empty string on failure. */
+            if (strlen($ret)) {
                 return $ret;
             }
         }
+
         return substr($string, $start, $length);
     }
 
