@@ -6,9 +6,9 @@
  * people *KNOW* HTML is a security hole, clean up what we can, and leave it
  * at that.
  *
- * $Horde: framework/Text_Filter/Filter/xss.php,v 1.1.2.16 2008/12/10 15:13:18 chuck Exp $
+ * $Horde: framework/Text_Filter/Filter/xss.php,v 1.1.2.18 2009/01/22 10:14:21 jan Exp $
  *
- * Copyright 2004-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2004-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -109,85 +109,85 @@ class Text_Filter_xss extends Text_Filter {
          * <applet>, <base>, etc. Meta refreshes and iframes, too. */
         $malicious = array(
             '/<([^>a-z]*)' .
-            '(s|&#0*83;?|&#x0*53;?|&#0*115;?|&#x0*73;?)\s*' .
-            '(c|&#0*67;?|&#x0*43;?|&#0*99;?|&#x0*63;?)\s*' .
-            '(r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)\s*' .
-            '(i|&#0*73;?|&#x0*49;?|&#0*105;?|&#x0*69;?)\s*' .
-            '(p|&#0*80;?|&#x0*50;?|&#0*112;?|&#x0*70;?)\s*' .
-            '(t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)\s*/i',
+            '(?:s|&#0*83;?|&#x0*53;?|&#0*115;?|&#x0*73;?)\s*' .
+            '(?:c|&#0*67;?|&#x0*43;?|&#0*99;?|&#x0*63;?)\s*' .
+            '(?:r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)\s*' .
+            '(?:i|&#0*73;?|&#x0*49;?|&#0*105;?|&#x0*69;?)\s*' .
+            '(?:p|&#0*80;?|&#x0*50;?|&#0*112;?|&#x0*70;?)\s*' .
+            '(?:t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)(\s*)/i',
 
             '/<([^>a-z]*)' .
-            '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
-            '(m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
-            '(b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
-            '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
-            '(d|&#0*68;?|&#0*44;?|&#0*100;?|&#0*64;?)\s*/i',
+            '(?:e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
+            '(?:m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
+            '(?:b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
+            '(?:e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
+            '(?:d|&#0*68;?|&#0*44;?|&#0*100;?|&#0*64;?)(\s*)/i',
 
             '/<([^>a-z]*)' .
-            '(x|&#0*88;?|&#0*58;?|&#0*120;?|&#0*78;?)\s*' .
-            '(m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
-            '(l|&#0*76;?|&#x0*4c;?|&#0*108;?|&#x0*6c;?)\s*/i',
+            '(?:x|&#0*88;?|&#0*58;?|&#0*120;?|&#0*78;?)\s*' .
+            '(?:m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
+            '(?:l|&#0*76;?|&#x0*4c;?|&#0*108;?|&#x0*6c;?)(\s*)/i',
 
             '/<([^>a-z]*)\?([^>a-z]*)' .
-            '(i|&#0*73;?|&#x0*49;?|&#0*105;?|&#x0*69;?)\s*' .
-            '(m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
-            '(p|&#0*80;?|&#x0*50;?|&#0*112;?|&#x0*70;?)\s*' .
-            '(o|&#0*79;?|&#0*4f;?|&#0*111;?|&#0*6f;?)\s*' .
-            '(r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)\s*' .
-            '(t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)\s*/i',
+            '(?:i|&#0*73;?|&#x0*49;?|&#0*105;?|&#x0*69;?)\s*' .
+            '(?:m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
+            '(?:p|&#0*80;?|&#x0*50;?|&#0*112;?|&#x0*70;?)\s*' .
+            '(?:o|&#0*79;?|&#0*4f;?|&#0*111;?|&#0*6f;?)\s*' .
+            '(?:r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)\s*' .
+            '(?:t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)(\s*)/i',
 
             '/<([^>a-z]*)' .
-            '(b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
-            '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
-            '(s|&#0*83;?|&#x0*53;?|&#0*115;?|&#x0*73;?)\s*' .
-            '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
+            '(?:b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
+            '(?:a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
+            '(?:s|&#0*83;?|&#x0*53;?|&#0*115;?|&#x0*73;?)\s*' .
+            '(?:e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)(\s*)' .
             '[^line]/i',
 
             '/<([^>a-z]*)' .
-            '(m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
-            '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
-            '(t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)\s*' .
-            '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*/i',
+            '(?:m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
+            '(?:e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
+            '(?:t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)\s*' .
+            '(?:a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)(\s*)/i',
 
             '/<([^>a-z]*)' .
-            '(j|&#0*74;?|&#0*4a;?|&#0*106;?|&#0*6a;?)\s*' .
-            '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
-            '(v|&#0*86;?|&#0*56;?|&#0*118;?|&#0*76;?)\s*' .
-            '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*/i',
+            '(?:j|&#0*74;?|&#0*4a;?|&#0*106;?|&#0*6a;?)\s*' .
+            '(?:a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
+            '(?:v|&#0*86;?|&#0*56;?|&#0*118;?|&#0*76;?)\s*' .
+            '(?:a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)(\s*)/i',
 
             '/<([^>a-z]*)' .
-            '(o|&#0*79;?|&#0*4f;?|&#0*111;?|&#0*6f;?)\s*' .
-            '(b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
-            '(j|&#0*74;?|&#0*4a;?|&#0*106;?|&#0*6a;?)\s*' .
-            '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
-            '(c|&#0*67;?|&#x0*43;?|&#0*99;?|&#x0*63;?)\s*' .
-            '(t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)\s*/i',
+            '(?:o|&#0*79;?|&#0*4f;?|&#0*111;?|&#0*6f;?)\s*' .
+            '(?:b|&#0*66;?|&#0*42;?|&#0*98;?|&#0*62;?)\s*' .
+            '(?:j|&#0*74;?|&#0*4a;?|&#0*106;?|&#0*6a;?)\s*' .
+            '(?:e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
+            '(?:c|&#0*67;?|&#x0*43;?|&#0*99;?|&#x0*63;?)\s*' .
+            '(?:t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)(\s*)/i',
 
             '/<([^>a-z]*)' .
-            '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
-            '(p|&#0*80;?|&#x0*50;?|&#0*112;?|&#x0*70;?)\s*' .
-            '(p|&#0*80;?|&#x0*50;?|&#0*112;?|&#x0*70;?)\s*' .
-            '(l|&#0*76;?|&#x0*4c;?|&#0*108;?|&#x0*6c;?)\s*' .
-            '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
-            '(t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)\s*/i',
+            '(?:a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
+            '(?:p|&#0*80;?|&#x0*50;?|&#0*112;?|&#x0*70;?)\s*' .
+            '(?:p|&#0*80;?|&#x0*50;?|&#0*112;?|&#x0*70;?)\s*' .
+            '(?:l|&#0*76;?|&#x0*4c;?|&#0*108;?|&#x0*6c;?)\s*' .
+            '(?:e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
+            '(?:t|&#0*84;?|&#x0*54;?|&#0*116;?|&#x0*74;?)(\s*)/i',
 
             '/<([^>a-z]*)' .
-            '(l|&#0*76;?|&#x0*4c;?|&#0*108;?|&#x0*6c;?)\s*' .
-            '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
-            '(y|&#0*89;?|&#0*59;?|&#0*121;?|&#0*79;?)\s*' .
-            '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
-            '(r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)\s*/i',
+            '(?:l|&#0*76;?|&#x0*4c;?|&#0*108;?|&#x0*6c;?)\s*' .
+            '(?:a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
+            '(?:y|&#0*89;?|&#0*59;?|&#0*121;?|&#0*79;?)\s*' .
+            '(?:e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*' .
+            '(?:r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)(\s*)/i',
 
             '/<([^>a-z]*)' .
-            '(i|&#0*73;?|&#x0*49;?|&#0*105;?|&#x0*69;?)?\s*' .
-            '(f|&#0*70;?|&#0*46;?|&#0*102;?|&#0*66;?)\s*' .
-            '(r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)\s*' .
-            '(a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
-            '(m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
-            '(e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)\s*/i');
+            '(?:i|&#0*73;?|&#x0*49;?|&#0*105;?|&#x0*69;?)?\s*' .
+            '(?:f|&#0*70;?|&#0*46;?|&#0*102;?|&#0*66;?)\s*' .
+            '(?:r|&#0*82;?|&#x0*52;?|&#0*114;?|&#x0*72;?)\s*' .
+            '(?:a|&#0*65;?|&#0*41;?|&#0*97;?|&#0*61;?)\s*' .
+            '(?:m|&#0*77;?|&#0*4d;?|&#0*109;?|&#0*6d;?)\s*' .
+            '(?:e|&#0*69;?|&#0*45;?|&#0*101;?|&#0*65;?)(\s*)/i');
 
         foreach ($malicious as $pattern) {
-            $patterns[$pattern] = '<' . $this->_params['replace'] . '_tag';
+            $patterns[$pattern] = '<$1' . $this->_params['replace'] . '_tag$2';
         }
 
         /* Comment out style/link tags. */
