@@ -12,9 +12,9 @@ include_once 'Horde/NLS.php';
 /**
  * Class representing iCalendar files.
  *
- * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.72 2008/10/28 23:40:30 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar.php,v 1.57.4.76 2009/05/14 10:47:29 jan Exp $
  *
- * Copyright 2003-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2003-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -592,7 +592,11 @@ class Horde_iCalendar {
         }
 
         // Unfold any folded lines.
-        $vCal = preg_replace('/[\r\n]+[ \t]/', '', $vCal);
+        if ($this->isOldFormat()) {
+            $vCal = preg_replace('/[\r\n]+([ \t])/', '$1', $vCal);
+        } else {
+            $vCal = preg_replace('/[\r\n]+[ \t]/', '', $vCal);
+        }
 
         // Parse the remaining attributes.
         if (preg_match_all('/^((?:[^":]+|(?:"[^"]*")+)*):([^\r\n]*)\r?$/m', $vCal, $matches)) {
@@ -604,7 +608,7 @@ class Horde_iCalendar {
 
                 // Parse parameters.
                 if (!empty($parts[2])) {
-                    preg_match_all('/;(([^;=]*)(=([^;]*))?)/', $parts[2], $param_parts);
+                    preg_match_all('/;(([^;=]*)(=("[^"]*"|[^;]*))?)/', $parts[2], $param_parts);
                     foreach ($param_parts[2] as $key => $paramName) {
                         $paramName = String::upper($paramName);
                         $paramValue = $param_parts[4][$key];
@@ -614,6 +618,9 @@ class Horde_iCalendar {
                                 $paramValue = $paramValue[0];
                             }
                         }
+                        if (preg_match('/"([^"]*)"/', $paramValue, $parts)) {
+                            $paramValue = $parts[1];
+                        }
                         $params[$paramName] = $paramValue;
                     }
                 }
@@ -645,6 +652,8 @@ class Horde_iCalendar {
                 case 'COMPLETED':
                 case 'CREATED':
                 case 'LAST-MODIFIED':
+                case 'X-MOZ-LASTACK': 
+                case 'X-MOZ-SNOOZE-TIME': 
                     $this->setAttribute($tag, $this->_parseDateTime($value, $tzid), $params);
                     break;
 
@@ -670,12 +679,9 @@ class Horde_iCalendar {
                     break;
 
                 case 'TRIGGER':
-                    if (isset($params['VALUE'])) {
-                        if ($params['VALUE'] == 'DATE-TIME') {
+                    if (isset($params['VALUE']) &&
+                        $params['VALUE'] == 'DATE-TIME') {
                             $this->setAttribute($tag, $this->_parseDateTime($value, $tzid), $params);
-                        } else {
-                            $this->setAttribute($tag, $this->_parseDuration($value), $params);
-                        }
                     } else {
                         $this->setAttribute($tag, $this->_parseDuration($value), $params);
                     }
@@ -839,7 +845,29 @@ class Horde_iCalendar {
                     if ($param_value === null) {
                         $params_str .= ";$param_name";
                     } else {
-                        $params_str .= ";$param_name=$param_value";
+                        $len = strlen($param_value);
+                        $safe_value = '';
+                        $quote = false;
+                        for ($i = 0; $i < $len; ++$i) {
+                            $ord = ord($param_value[$i]);
+                            // Accept only valid characters.
+                            if ($ord == 9 || $ord == 32 || $ord == 33 ||
+                                ($ord >= 35 && $ord <= 126) ||
+                                $ord >= 128) {
+                                $safe_value .= $param_value[$i];
+                                // Characters above 128 do not need to be
+                                // quoted as per RFC2445 but Outlook requires
+                                // this.
+                                if ($ord == 44 || $ord == 58 || $ord == 59 ||
+                                    $ord >= 128) {
+                                    $quote = true;
+                                }
+                            }
+                        }
+                        if ($quote) {
+                            $safe_value = '"' . $safe_value . '"';
+                        }
+                        $params_str .= ";$param_name=$safe_value";
                     }
                 }
             }
@@ -851,6 +879,8 @@ class Horde_iCalendar {
             case 'CREATED':
             case 'DCREATED':
             case 'LAST-MODIFIED':
+            case 'X-MOZ-LASTACK':
+            case 'X-MOZ-SNOOZE-TIME':
                 $value = $this->_exportDateTime($value);
                 break;
 
