@@ -2,9 +2,9 @@
 /**
  * Class representing vJournals.
  *
- * $Horde: framework/iCalendar/iCalendar/vjournal.php,v 1.8.10.5 2007/01/02 13:54:51 jan Exp $
+ * $Horde: framework/iCalendar/iCalendar/vjournal.php,v 1.8.10.9 2009/01/06 15:23:53 jan Exp $
  *
- * Copyright 2003-2007 Mike Cochrane <mike@graftonhall.co.nz>
+ * Copyright 2003-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -20,11 +20,6 @@ class Horde_iCalendar_vjournal extends Horde_iCalendar {
         return 'vJournal';
     }
 
-    function parsevCalendar($data)
-    {
-        parent::parsevCalendar($data, 'VJOURNAL');
-    }
-
     function exportvCalendar()
     {
         return parent::_exportvData('VJOURNAL');
