@@ -9,9 +9,9 @@
  * 'apache', or 'www-data') MUST have read/write permission to the
  * directory you specify as the 'vfsroot'.
  *
- * $Horde: framework/VFS/VFS/file.php,v 1.63.4.22 2007/01/02 13:54:48 jan Exp $
+ * $Horde: framework/VFS/lib/VFS/file.php,v 1.1.2.5 2009/01/06 15:23:47 jan Exp $
  *
- * Copyright 2002-2007 Chuck Hagenbuch <chuck@horde.org>
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -77,26 +77,50 @@ class VFS_file extends VFS {
      */
     function read($path, $name)
     {
-        $file = $this->_getNativePath($path, $name);
-        if (function_exists('file_get_contents')) {
-            $data = file_get_contents($file);
-        } else {
-            $fp = @fopen($file, 'rb');
-            if (!$fp) {
-                return PEAR::raiseError(_("Unable to open VFS file."));
-            }
-            $filesize = filesize($file);
-            if ($filesize == 0) {
-                $data = '';
-            } else {
-                $data = fread($fp, $filesize);
-            }
-            fclose($fp);
+        $data = @file_get_contents($this->_getNativePath($path, $name));
+        if ($data === false) {
+            return PEAR::raiseError(_("Unable to open VFS file."));
         }
 
         return $data;
     }
 
+    /**
+     * Retrieves a file from the VFS as an on-disk local file.
+     *
+     * This function provides a file on local disk with the data of a VFS file
+     * in it. This file <em>cannot</em> be modified! The behavior if you do
+     * modify it is undefined. It will be removed at the end of the request.
+     *
+     * @param string $path  The pathname to the file.
+     * @param string $name  The filename to retrieve.
+     *
+     * @return string A local filename.
+     */
+    function readFile($path, $name)
+    {
+        return $this->_getNativePath($path, $name);
+    }
+
+    /**
+     * Open a read-only stream to a file in the VFS.
+     *
+     * @param string $path  The pathname to the file.
+     * @param string $name  The filename to retrieve.
+     *
+     * @return resource  The stream.
+     */
+    function readStream($path, $name)
+    {
+        $mode = OS_WINDOWS ? 'rb' : 'r';
+        $stream = @fopen($this->_getNativePath($path, $name), $mode);
+        if (!is_resource($stream)) {
+            return PEAR::raiseError(_("Unable to open VFS file."));
+        }
+
+        return $stream;
+    }
+
     /**
      * Retrieves a part of a file from the VFS. Particularly useful when
      * reading large files which would exceed the PHP memory limits if they
@@ -193,19 +217,31 @@ class VFS_file extends VFS {
     /**
      * Moves a file in the database and the file system.
      *
-     * @param string $path  The path to store the file in.
-     * @param string $name  The filename to use.
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param string $dest         The destination of the file.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function move($path, $name, $dest)
+    function move($path, $name, $dest, $autocreate = false)
     {
         $orig = $this->_getNativePath($path, $name);
         if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
             return PEAR::raiseError(_("Cannot move file(s) - destination is within source."));
         }
 
+        if ($autocreate) {
+            $result = $this->autocreatePath($dest);
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
+        }
+
         $fileCheck = $this->listFolder($dest, false);
+        if (is_a($fileCheck, 'PEAR_Error')) {
+            return $fileCheck;
+        }
         foreach ($fileCheck as $file) {
             if ($file['name'] == $name) {
                 return PEAR::raiseError(_("Unable to move VFS file."));
@@ -222,20 +258,31 @@ class VFS_file extends VFS {
     /**
      * Copies a file through the backend.
      *
-     * @param string $path  The path to store the file in.
-     * @param string $name  The filename to use.
-     * @param string $dest  The destination of the file.
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param string $dest         The destination of the file.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function copy($path, $name, $dest)
+    function copy($path, $name, $dest, $autocreate = false)
     {
         $orig = $this->_getNativePath($path, $name);
         if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
             return PEAR::raiseError(_("Cannot copy file(s) - source and destination are the same."));
         }
 
+        if ($autocreate) {
+            $result = $this->autocreatePath($dest);
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
+        }
+
         $fileCheck = $this->listFolder($dest, false);
+        if (is_a($fileCheck, 'PEAR_Error')) {
+            return $fileCheck;
+        }
         foreach ($fileCheck as $file) {
             if ($file['name'] == $name) {
                 return PEAR::raiseError(_("Unable to copy VFS file."));
@@ -288,13 +335,19 @@ class VFS_file extends VFS {
         }
 
         // Otherwise we go ahead and try to write out the file.
-        $fp = @fopen($this->_getNativePath($path, $name), 'w');
-        if (!$fp) {
-            return PEAR::raiseError(_("Unable to open VFS file for writing."));
-        }
+        if (function_exists('file_put_contents')) {
+            if (!@file_put_contents($this->_getNativePath($path, $name), $data)) {
+                return PEAR::raiseError(_("Unable to write VFS file data."));
+            }
+        } else {
+            $fp = @fopen($this->_getNativePath($path, $name), 'w');
+            if (!$fp) {
+                return PEAR::raiseError(_("Unable to open VFS file for writing."));
+            }
 
-        if (!@fwrite($fp, $data)) {
-            return PEAR::raiseError(_("Unable to write VFS file data."));
+            if (!@fwrite($fp, $data)) {
+                return PEAR::raiseError(_("Unable to write VFS file data."));
+            }
         }
 
         return true;
@@ -359,7 +412,7 @@ class VFS_file extends VFS {
     /**
      * Creates a folder on the VFS.
      *
-     * @param string $path  The path to delete the folder from.
+     * @param string $path  The path to create the folder in.
      * @param string $name  The foldername to use.
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
@@ -456,8 +509,10 @@ class VFS_file extends VFS {
             // Group
             $file['group'] = filegroup($entry);
             if (function_exists('posix_getgrgid')) {
-                $group = posix_getgrgid($file['group']);
-                $file['group'] = $group['name'];
+                if (PHP_VERSION != '5.2.1') {
+                    $group = posix_getgrgid($file['group']);
+                    $file['group'] = $group['name'];
+                }
             }
 
             // Size
@@ -635,6 +690,19 @@ class VFS_file extends VFS {
         return true;
     }
 
+    /**
+     * Returns if a given file or folder exists in a folder.
+     *
+     * @param string $path  The path to the folder.
+     * @param string $name  The file or folder name.
+     *
+     * @return boolean  True if it exists, false otherwise.
+     */
+    function exists($path, $name)
+    {
+        return file_exists($this->_getNativePath($path, $name));
+    }
+
     /**
      * Return a full filename on the native filesystem, from a VFS
      * path and name.
@@ -649,14 +717,14 @@ class VFS_file extends VFS {
     function _getNativePath($path = '', $name = '')
     {
         $name = basename($name);
-        if (!empty($name)) {
+        if (strlen($name)) {
             $name = str_replace('..', '', $name);
             if (substr($name, 0, 1) != '/') {
                 $name = '/' . $name;
             }
         }
 
-        if (!empty($path)) {
+        if (strlen($path)) {
             if (isset($this->_params['home']) &&
                 preg_match('|^~/?(.*)$|', $path, $matches)) {
                 $path = $this->_params['home'] . '/' . $matches[1];
@@ -684,7 +752,7 @@ class VFS_file extends VFS {
     function _connect()
     {
         if ((@is_dir($this->_params['vfsroot']) &&
-             @is_readable($this->_params['vfsroot'])) ||
+             is_readable($this->_params['vfsroot'])) ||
             @mkdir($this->_params['vfsroot'])) {
             return true;
         } else {
