@@ -2,31 +2,42 @@
 /**
  * VFS implementation for an FTP server.
  *
- * <pre>
- * Required values for $params:
+ * Required values for $params:<pre>
  *      'username'       The username with which to connect to the ftp server.
  *      'password'       The password with which to connect to the ftp server.
- *      'hostspec'       The ftp server to connect to.
- * Optional values for $params:
+ *      'hostspec'       The ftp server to connect to.</pre>
+ *
+ * Optional values for $params:<pre>
+ *      'lsformat'       The return formatting from the 'ls' command).
+ *                       Values: 'aix', 'standard' (default)
+ *      'maplocalids'    If true and the POSIX extension is available, the
+ *                       driver will map the user and group IDs returned from
+ *                       the FTP server with the local IDs from the local
+ *                       password file.  This is useful only if the FTP server
+ *                       is running on localhost or if the local user/group
+ *                       IDs are identical to the remote FTP server.
  *      'pasv'           If true, connection will be set to passive mode.
  *      'port'           The port used to connect to the ftp server if other
  *                       than 21.
  *      'ssl'            If true, and PHP had been compiled with OpenSSL
  *                       support, TLS transport-level encryption will be
  *                       negotiated with the server.
- * </pre>
+ *      'timeout'        If defined, use this value as the timeout for the
+ *                       server.
+ *      'type'           The type of the remote FTP server.
+ *                       Possible values: 'unix', 'win', 'netware'
+ *                       By default, we attempt to auto-detect type.</pre>
  *
- * $Horde: framework/VFS/VFS/ftp.php,v 1.75.4.4 2005/02/17 22:09:10 jan Exp $
+ * $Horde: framework/VFS/lib/VFS/ftp.php,v 1.1.2.6 2009/01/06 15:23:47 jan Exp $
  *
- * Copyright 2002-2005 Chuck Hagenbuch <chuck@horde.org>
- * Copyright 2002-2005 Michael Varghese <mike.varghese@ascellatech.com>
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
+ * Copyright 2002-2007 Michael Varghese <mike.varghese@ascellatech.com>
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
  *
  * @author  Chuck Hagenbuch <chuck@horde.org>
  * @author  Michael Varghese <mike.varghese@ascellatech.com>
- * @since   Horde 2.2
  * @package VFS
  */
 class VFS_ftp extends VFS {
@@ -34,14 +45,14 @@ class VFS_ftp extends VFS {
     /**
      * List of additional credentials required for this VFS backend.
      *
-     * @var array $_credentials
+     * @var array
      */
     var $_credentials = array('username', 'password');
 
     /**
      * List of permissions and if they can be changed in this VFS backend.
      *
-     * @var array $_permissions
+     * @var array
      */
     var $_permissions = array(
         'owner' => array('read' => true, 'write' => true, 'execute' => true),
@@ -51,57 +62,137 @@ class VFS_ftp extends VFS {
     /**
      * Variable holding the connection to the ftp server.
      *
-     * @var resource $_stream
+     * @var resource
      */
     var $_stream = false;
 
     /**
-     * Retrieves a file from the VFS.
+     * Local cache array for user IDs.
+     *
+     * @var array
+     */
+    var $_uids = array();
+
+    /**
+     * Local cache array for group IDs.
+     *
+     * @var array
+     */
+    var $_gids = array();
+
+    /**
+     */
+    var $_type;
+
+    /**
+     * Returns the size of a file.
      *
      * @access public
      *
+     * @param string $path  The path of the file.
+     * @param string $name  The filename.
+     *
+     * @return integer  The size of the file in bytes or PEAR_Error on
+     *                  failure.
+     */
+    function size($path, $name)
+    {
+        $conn = $this->_connect();
+        if (is_a($conn, 'PEAR_Error')) {
+            return $conn;
+        }
+
+        if (($size = @ftp_size($this->_stream, $this->_getPath($path, $name))) === false) {
+            return PEAR::raiseError(sprintf(_("Unable to check file size of \"%s\"."), $this->_getPath($path, $name)));
+        }
+
+        return $size;
+    }
+
+    /**
+     * Retrieves a file from the VFS.
+     *
      * @param string $path  The pathname to the file.
      * @param string $name  The filename to retrieve.
      *
      * @return string  The file data.
      */
     function read($path, $name)
+    {
+        $file = $this->readFile($path, $name);
+        if (is_a($file, 'PEAR_Error')) {
+            return $file;
+        }
+
+        $size = filesize($file);
+        if ($size === 0) {
+            return '';
+        }
+
+        return file_get_contents($file);
+    }
+
+    /**
+     * Retrieves a file from the VFS as an on-disk local file.
+     *
+     * This function provides a file on local disk with the data of a VFS file
+     * in it. This file <em>cannot</em> be modified! The behavior if you do
+     * modify it is undefined. It will be removed at the end of the request.
+     *
+     * @param string $path  The pathname to the file.
+     * @param string $name  The filename to retrieve.
+     *
+     * @return string A local filename.
+     */
+    function readFile($path, $name)
     {
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
 
-        $tmpFile = $this->_getTempFile();
-        $fetch = @ftp_get($this->_stream, $tmpFile,
-                          $this->_getPath($path, $name), FTP_BINARY);
-        if ($fetch === false) {
+        // Create a temporary file and register it for deletion at the
+        // end of this request.
+        $localFile = $this->_getTempFile();
+        if (!$localFile) {
+            return PEAR::raiseError(_("Unable to create temporary file."));
+        }
+        register_shutdown_function(create_function('', '@unlink(\'' . addslashes($localFile) . '\');'));
+
+        $result = @ftp_get(
+            $this->_stream,
+            $localFile,
+            $this->_getPath($path, $name),
+            FTP_BINARY);
+        if ($result === false) {
             return PEAR::raiseError(sprintf(_("Unable to open VFS file \"%s\"."), $this->_getPath($path, $name)));
         }
 
-        $size = filesize($tmpFile);
-        if ($size === 0) {
-            return '';
+        return $localFile;
+    }
+
+    /**
+     * Open a stream to a file in the VFS.
+     *
+     * @param string $path  The pathname to the file.
+     * @param string $name  The filename to retrieve.
+     *
+     * @return resource  The stream.
+     */
+    function readStream($path, $name)
+    {
+        $file = $this->readFile($path, $name);
+        if (is_a($file, 'PEAR_Error')) {
+            return $file;
         }
 
-        if (OS_WINDOWS) {
-            $mode = 'rb';
-        } else {
-            $mode = 'r';
-        }
-        $fp = fopen($tmpFile, $mode);
-        $data = fread($fp, $size);
-        fclose($fp);
-        unlink($tmpFile);
-
-        return $data;
+        $mode = OS_WINDOWS ? 'rb' : 'r';
+        return fopen($file, $mode);
     }
 
     /**
      * Stores a file in the VFS.
      *
-     * @access public
-     *
      * @param string $path         The path to store the file in.
      * @param string $name         The filename to use.
      * @param string $tmpFile      The temporary file containing the data to
@@ -117,6 +208,11 @@ class VFS_ftp extends VFS {
             return $conn;
         }
 
+        $res = $this->_checkQuotaWrite('file', $tmpFile);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         if (!@ftp_put($this->_stream, $this->_getPath($path, $name), $tmpFile, FTP_BINARY)) {
             if ($autocreate) {
                 $result = $this->autocreatePath($path);
@@ -137,8 +233,6 @@ class VFS_ftp extends VFS {
     /**
      * Stores a file in the VFS from raw data.
      *
-     * @access public
-     *
      * @param string $path         The path to store the file in.
      * @param string $name         The filename to use.
      * @param string $data         The file data.
@@ -148,6 +242,11 @@ class VFS_ftp extends VFS {
      */
     function writeData($path, $name, $data, $autocreate = false)
     {
+        $res = $this->_checkQuotaWrite('string', $data);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         $tmpFile = $this->_getTempFile();
         $fp = fopen($tmpFile, 'wb');
         fwrite($fp, $data);
@@ -161,8 +260,6 @@ class VFS_ftp extends VFS {
     /**
      * Deletes a file from the VFS.
      *
-     * @access public
-     *
      * @param string $path  The path to delete the file from.
      * @param string $name  The filename to delete.
      *
@@ -170,6 +267,11 @@ class VFS_ftp extends VFS {
      */
     function deleteFile($path, $name)
     {
+        $res = $this->_checkQuotaDelete($path, $name);
+        if (is_a($res, 'PEAR_Error')) {
+            return $res;
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
@@ -185,8 +287,6 @@ class VFS_ftp extends VFS {
     /**
      * Checks if a given item is a folder.
      *
-     * @access public
-     *
      * @param string $path  The parent folder.
      * @param string $name  The item name.
      *
@@ -215,8 +315,6 @@ class VFS_ftp extends VFS {
     /**
      * Deletes a folder from the VFS.
      *
-     * @access public
-     *
      * @param string $path        The parent folder.
      * @param string $name        The name of the folder to delete.
      * @param boolean $recursive  Force a recursive delete?
@@ -276,8 +374,6 @@ class VFS_ftp extends VFS {
     /**
      * Renames a file in the VFS.
      *
-     * @access public
-     *
      * @param string $oldpath  The old path to the file.
      * @param string $oldname  The old filename.
      * @param string $newpath  The new path of the file.
@@ -287,11 +383,14 @@ class VFS_ftp extends VFS {
      */
     function rename($oldpath, $oldname, $newpath, $newname)
     {
-        $conn = $this->_connect();
-        if (is_a($conn, 'PEAR_Error')) {
+        if (is_a($conn = $this->_connect(), 'PEAR_Error')) {
             return $conn;
         }
 
+        if (is_a($result = $this->autocreatePath($newpath), 'PEAR_Error')) {
+            return $result;
+        }
+
         if (!@ftp_rename($this->_stream, $this->_getPath($oldpath, $oldname), $this->_getPath($newpath, $newname))) {
             return PEAR::raiseError(sprintf(_("Unable to rename VFS file \"%s\"."), $this->_getPath($oldpath, $oldname)));
         }
@@ -302,8 +401,6 @@ class VFS_ftp extends VFS {
     /**
      * Creates a folder on the VFS.
      *
-     * @access public
-     *
      * @param string $path  The parent folder.
      * @param string $name  The name of the new folder.
      *
@@ -326,9 +423,6 @@ class VFS_ftp extends VFS {
     /**
      * Changes permissions for an item on the VFS.
      *
-     * @access public
-     * @abstract
-     *
      * @param string $path        The parent folder of the item.
      * @param string $name        The name of the item.
      * @param string $permission  The permission to set.
@@ -352,9 +446,6 @@ class VFS_ftp extends VFS {
     /**
      * Returns an an unsorted file list of the specified directory.
      *
-     * @access public
-     * @abstract
-     *
      * @param string $path       The path of the directory.
      * @param mixed $filter      String/hash to filter file/dirname on.
      * @param boolean $dotfiles  Show dotfiles?
@@ -371,23 +462,34 @@ class VFS_ftp extends VFS {
         }
 
         $files = array();
-        $type = VFS::strtolower(@ftp_systype($this->_stream));
-        if ($type == 'unknown') {
-            // Go with unix-style listings by default.
-            $type = 'unix';
-        } elseif (strpos($type, 'win') !== false) {
-            $type = 'win';
+
+        if (empty($this->_type)) {
+            if (!empty($this->_params['type'])) {
+                $this->_type = $this->_params['type'];
+            } else {
+                $type = VFS::strtolower(@ftp_systype($this->_stream));
+                if ($type == 'unknown') {
+                    // Go with unix-style listings by default.
+                    $type = 'unix';
+                } elseif (strpos($type, 'win') !== false) {
+                    $type = 'win';
+                } elseif (strpos($type, 'netware') !== false) {
+                    $type = 'netware';
+                }
+
+                $this->_type = $type;
+            }
         }
 
         $olddir = $this->getCurrentDirectory();
-        if (!empty($path)) {
+        if (strlen($path)) {
             $res = $this->_setPath($path);
             if (is_a($res, 'PEAR_Error')) {
                 return $res;
             }
         }
 
-        if ($type == 'unix') {
+        if ($this->_type == 'unix') {
             // If we don't want dotfiles, We can save work here by not
             // doing an ls -a and then not doing the check later (by
             // setting $dotfiles to true, the if is short-circuited).
@@ -411,17 +513,47 @@ class VFS_ftp extends VFS {
             return array();
         }
 
+        /* If 'maplocalids' is set, check for the POSIX extension. */
+        $mapids = false;
+        if (!empty($this->_params['maplocalids']) &&
+            extension_loaded('posix')) {
+            $mapids = true;
+        }
+
+        $currtime = time();
+
         foreach ($list as $line) {
             $file = array();
             $item = preg_split('/\s+/', $line);
-            if ($type == 'unix' || ($type == 'win' && !preg_match('|\d\d-\d\d-\d\d|', $item[0]))) {
+            if (($this->_type == 'unix') ||
+                (($this->_type == 'win') && !preg_match('|\d\d-\d\d-\d\d|', $item[0]))) {
                 if (count($item) < 8 || substr($line, 0, 5) == 'total') {
                     continue;
                 }
                 $file['perms'] = $item[0];
-                $file['owner'] = $item[2];
-                $file['group'] = $item[3];
-                $file['name'] = substr($line, strpos($line, sprintf("%s %2s %5s", $item[5], $item[6], $item[7])) + 13);
+                if ($mapids) {
+                    if (!isset($this->_uids[$item[2]])) {
+                        $entry = posix_getpwuid($item[2]);
+                        $this->_uids[$item[2]] = (empty($entry)) ? $item[2] : $entry['name'];
+                    }
+                    $file['owner'] = $this->_uids[$item[2]];
+                    if (!isset($this->_uids[$item[3]])) {
+                        $entry = posix_getgrgid($item[3]);
+                        $this->_uids[$item[3]] = (empty($entry)) ? $item[3] : $entry['name'];
+                    }
+                    $file['group'] = $this->_uids[$item[3]];
+
+                } else {
+                    $file['owner'] = $item[2];
+                    $file['group'] = $item[3];
+                }
+
+                if (!empty($this->_params['lsformat']) &&
+                    ($this->_params['lsformat'] == 'aix')) {
+                    $file['name'] = substr($line, strpos($line, sprintf("%s %2s %-5s", $item[5], $item[6], $item[7])) + 13);
+                } else {
+                    $file['name'] = substr($line, strpos($line, sprintf("%s %2s %5s", $item[5], $item[6], $item[7])) + 13);
+                }
 
                 // Filter out '.' and '..' entries.
                 if (preg_match('/^\.\.?\/?$/', $file['name'])) {
@@ -466,13 +598,47 @@ class VFS_ftp extends VFS {
                     $file['size'] = $item[4];
                 }
                 if (strpos($item[7], ':') !== false) {
-                    $file['date'] = strtotime($item[7] . ':00' . $item[5] . ' ' . $item[6] . ' ' . date('Y', time()));
-                    if ($file['date'] > time()) {
-                        $file['date'] = strtotime($item[7] . ':00' . $item[5] . ' ' . $item[6] . ' ' . (date('Y', time()) - 1));
+                    $file['date'] = strtotime($item[7] . ':00' . $item[5] . ' ' . $item[6] . ' ' . date('Y', $currtime));
+                    // If the ftp server reports a file modification date more
+                    // less than one day in the future, don't try to subtract
+                    // a year from the date.  There is no way to know, for
+                    // example, if the VFS server and the ftp server reside
+                    // in different timezones.  We should simply report to the
+                    //  user what the FTP server is returning.
+                    if ($file['date'] > ($currtime + 86400)) {
+                        $file['date'] = strtotime($item[7] . ':00' . $item[5] . ' ' . $item[6] . ' ' . (date('Y', $currtime) - 1));
                     }
                 } else {
                     $file['date'] = strtotime('00:00:00' . $item[5] . ' ' . $item[6] . ' ' . $item[7]);
                 }
+            } elseif ($this->_type == 'netware') {
+                if (count($item) < 8 || substr($line, 0, 5) == 'total') {
+                    continue;
+                }
+
+                $file = array();
+                $file['perms'] = $item[1];
+                $file['owner'] = $item[2];
+                if ($item[0] == 'd') {
+                    $file['type'] = '**dir';
+                } else {
+                    $file['type'] = '**none';
+                }
+                $file['size'] = $item[3];
+
+                // We don't know the timezone here. Just report what the FTP server says.
+                if (strpos($item[6], ':') !== false) {
+                    $file['date'] = strtotime($item[6] . ':00 ' . $item[5] . ' ' . $item[4] . ' ' . date('Y'));
+                } else {
+                    $file['date'] = strtotime('00:00:00 ' . $item[5] . ' ' . $item[4] . ' ' . $item[6]);
+                }
+
+                $file['name'] = $item[7];
+                $index = 8;
+                while ($index < count($item)) {
+                    $file['name'] .= ' ' . $item[$index];
+                    $index++;
+                }
             } else {
                 /* Handle Windows FTP servers returning DOS-style file
                  * listings. */
@@ -526,9 +692,6 @@ class VFS_ftp extends VFS {
     /**
      * Returns a sorted list of folders in the specified directory.
      *
-     * @access public
-     * @abstract
-     *
      * @param string $path         The path of the directory to get the
      *                             directory list for.
      * @param mixed $filter        Hash of items to filter based on folderlist.
@@ -572,21 +735,32 @@ class VFS_ftp extends VFS {
     /**
      * Copies a file through the backend.
      *
-     * @access public
-     *
-     * @param string $path  The path of the original file.
-     * @param string $name  The name of the original file.
-     * @param string $dest  The name of the destination directory.
+     * @param string $path         The path of the original file.
+     * @param string $name         The name of the original file.
+     * @param string $dest         The name of the destination directory.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function copy($path, $name, $dest)
+    function copy($path, $name, $dest, $autocreate = false)
     {
+        $orig = $this->_getPath($path, $name);
+        if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
+            return PEAR::raiseError(_("Cannot copy file(s) - source and destination are the same."));
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
 
+        if ($autocreate) {
+            $result = $this->autocreatePath($dest);
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
+        }
+
         $fileCheck = $this->listFolder($dest, null, true);
         if (is_a($fileCheck, 'PEAR_Error')) {
             return $fileCheck;
@@ -597,38 +771,16 @@ class VFS_ftp extends VFS {
             }
         }
 
-        $isDir = false;
-        $dirCheck = $this->listFolder($path, null, false);
-        if (is_a($dirCheck, 'PEAR_Error')) {
-            return $dirCheck;
-        }
-        foreach ($dirCheck as $file) {
-            if ($file['name'] == $name && $file['type'] == '**dir') {
-                $isDir = true;
-                break;
-            }
-        }
-
-        if ($isDir) {
-            $result = $this->createFolder($dest, $name);
-
-            if (is_a($result, 'PEAR_Error')) {
+        if ($this->isFolder($path, $name)) {
+            if (is_a($result = $this->_copyRecursive($path, $name, $dest), 'PEAR_Error')) {
                 return $result;
             }
-
-            $file_list = $this->listFolder($this->_getPath($path, $name));
-            foreach ($file_list as $file) {
-                $result = $this->copy($this->_getPath($path, $name), $file['name'], $this->_getPath($dest, $name));
-                if (is_a($result, 'PEAR_Error')) {
-                    return $result;
-                }
-            }
         } else {
             $tmpFile = $this->_getTempFile();
-            $fetch = @ftp_get($this->_stream, $tmpFile, $this->_getPath($path, $name), FTP_BINARY);
+            $fetch = @ftp_get($this->_stream, $tmpFile, $orig, FTP_BINARY);
             if (!$fetch) {
                 unlink($tmpFile);
-                return PEAR::raiseError(sprintf(_("Failed to copy from \"%s\"."), $this->_getPath($path, $name)));
+                return PEAR::raiseError(sprintf(_("Failed to copy from \"%s\"."), $orig));
             }
 
             if (!@ftp_put($this->_stream, $this->_getPath($dest, $name), $tmpFile, FTP_BINARY)) {
@@ -645,29 +797,43 @@ class VFS_ftp extends VFS {
     /**
      * Moves a file through the backend.
      *
-     * @access public
-     *
-     * @param string $path  The path of the original file.
-     * @param string $name  The name of the original file.
-     * @param string $dest  The destination file name.
+     * @param string $path         The path of the original file.
+     * @param string $name         The name of the original file.
+     * @param string $dest         The destination file name.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function move($path, $name, $dest)
+    function move($path, $name, $dest, $autocreate = false)
     {
+        $orig = $this->_getPath($path, $name);
+        if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
+            return PEAR::raiseError(_("Cannot move file(s) - destination is within source."));
+        }
+
         $conn = $this->_connect();
         if (is_a($conn, 'PEAR_Error')) {
             return $conn;
         }
 
+        if ($autocreate) {
+            $result = $this->autocreatePath($dest);
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
+        }
+
         $fileCheck = $this->listFolder($dest, null, true);
+        if (is_a($fileCheck, 'PEAR_Error')) {
+            return $fileCheck;
+        }
         foreach ($fileCheck as $file) {
             if ($file['name'] == $name) {
                 return PEAR::raiseError(sprintf(_("%s already exists."), $this->_getPath($dest, $name)));
             }
         }
 
-        if (!@ftp_rename($this->_stream, $this->_getPath($path, $name), $this->_getPath($dest, $name))) {
+        if (!@ftp_rename($this->_stream, $orig, $this->_getPath($dest, $name))) {
             return PEAR::raiseError(sprintf(_("Failed to move to \"%s\"."), $this->_getPath($dest, $name)));
         }
 
@@ -677,8 +843,6 @@ class VFS_ftp extends VFS {
     /**
      * Returns the current working directory on the FTP server.
      *
-     * @access public
-     *
      * @return string  The current working directory.
      */
     function getCurrentDirectory()
@@ -686,7 +850,7 @@ class VFS_ftp extends VFS {
         if (is_a($connected = $this->_connect(), 'PEAR_Error')) {
             return $connected;
         }
-        return ftp_pwd($this->_stream);
+        return @ftp_pwd($this->_stream);
     }
 
     /**
@@ -706,25 +870,6 @@ class VFS_ftp extends VFS {
         return true;
     }
 
-    /**
-     * Returns the full path of an item.
-     *
-     * @access private
-     *
-     * @param string $path  The directory of the item.
-     * @param string $name  The name of the item.
-     *
-     * @return mixed  Full path to the file when $path is not empty and just
-     *                $name when not set.
-     */
-    function _getPath($path, $name)
-    {
-        if ($path !== '') {
-             return ($path . '/' . $name);
-        }
-        return ($name);
-    }
-
     /**
      * Returns the parent directory of the specified path.
      *
@@ -776,7 +921,7 @@ class VFS_ftp extends VFS {
             $required = array('hostspec', 'username', 'password');
             foreach ($required as $val) {
                 if (!isset($this->_params[$val])) {
-                    return PEAR::raiseError(sprintf(_("Required '%s' not specified in VFS configuration."), $val));
+                    return PEAR::raiseError(sprintf(_("Required \"%s\" not specified in VFS configuration."), $val));
                 }
             }
 
@@ -796,87 +941,21 @@ class VFS_ftp extends VFS {
 
             $connected = @ftp_login($this->_stream, $this->_params['username'], $this->_params['password']);
             if (!$connected) {
-                $this->_disconnect();
+                @ftp_quit($this->_stream);
+                $this->_stream = false;
                 return PEAR::raiseError(_("Authentication to FTP server failed."));
             }
 
             if (!empty($this->_params['pasv'])) {
                 @ftp_pasv($this->_stream, true);
             }
+
+            if (!empty($this->_params['timeout'])) {
+                ftp_set_option($this->_stream, FTP_TIMEOUT_SEC, $this->_params['timeout']);
+            }
         }
 
         return true;
     }
 
-    /**
-     * Disconnects from the FTP server and cleans up the connection.
-     *
-     * @access private
-     */
-    function _disconnect()
-    {
-        @ftp_quit($this->_stream);
-        $this->_stream = false;
-    }
-
-    /**
-     * Determines the location of the system temporary directory.  If a
-     * specific setting cannot be found, it defaults to /tmp
-     *
-     * @access private
-     *
-     * @return string  A directory name which can be used for temp files.
-     *                 Returns false if one could not be found.
-     */
-    function _getTempDir()
-    {
-        $tmp_locations = array('/tmp', '/var/tmp', 'c:\WUTemp', 'c:\temp', 'c:\windows\temp', 'c:\winnt\temp');
-
-        /* Try PHP's upload_tmp_dir directive. */
-        $tmp = ini_get('upload_tmp_dir');
-
-        /* Otherwise, try to determine the TMPDIR environment variable. */
-        if (empty($tmp)) {
-            $tmp = getenv('TMPDIR');
-        }
-
-        /* If we still cannot determine a value, then cycle through a list of
-         * preset possibilities. */
-        while (empty($tmp) && sizeof($tmp_locations)) {
-            $tmp_check = array_shift($tmp_locations);
-            if (@is_dir($tmp_check)) {
-                $tmp = $tmp_check;
-            }
-        }
-
-        /* If it is still empty, we have failed, so return false; otherwise
-         * return the directory determined. */
-        return empty($tmp) ? false : $tmp;
-    }
-
-    /**
-     * Creates a temporary file.
-     *
-     * @access private
-     *
-     * @return string  Returns the full path-name to the temporary file.
-     *                 Returns false if a temp file could not be created.
-     */
-    function _getTempFile()
-    {
-        $tmp_dir = $this->_getTempDir();
-        if (empty($tmp_dir)) {
-            return false;
-        }
-
-        $tmp_file = tempnam($tmp_dir, 'vfs');
-
-        /* If the file was created, then register it for deleton and return */
-        if (empty($tmp_file)) {
-            return false;
-        } else {
-            return $tmp_file;
-        }
-    }
-
 }
