@@ -1,4 +1,9 @@
 <?php
+/**
+ * $Horde: framework/VFS/lib/VFS/sql_file.php,v 1.1.2.2 2009/02/13 05:45:19 chuck Exp $
+ *
+ * @package VFS
+ */
 
 /**
  * File value for vfs_type column.
@@ -10,6 +15,11 @@ define('VFS_FILE', 1);
  */
 define('VFS_FOLDER', 2);
 
+/**
+ * VFS_file parent class.
+ */
+include_once 'VFS/file.php';
+
 /**
  * VFS:: implementation using PHP's PEAR database abstraction
  * layer and local file system for file storage.
@@ -36,12 +46,12 @@ define('VFS_FOLDER', 2);
  * The table structure for the VFS can be found in
  * data/vfs.sql.
  *
- * $Horde: framework/VFS/VFS/sql_file.php,v 1.44.4.22 2006/05/31 04:50:02 slusarz Exp $
+ * $Horde: framework/VFS/lib/VFS/sql_file.php,v 1.1.2.2 2009/02/13 05:45:19 chuck Exp $
  *
  * @author  Michael Varghese <mike.varghese@ascellatech.com>
  * @package VFS
  */
-class VFS_sql_file extends VFS {
+class VFS_sql_file extends VFS_file {
 
     /**
      * Handle for the current database connection.
@@ -50,57 +60,6 @@ class VFS_sql_file extends VFS {
      */
     var $_db = false;
 
-    /**
-     * Returns the size of a file.
-     *
-     * @access public
-     *
-     * @param string $path  The path of the file.
-     * @param string $name  The filename.
-     *
-     * @return integer  The size of the file in bytes or PEAR_Error on
-     *                  failure.
-     */
-    function size($path, $name)
-    {
-        if (($size = @filesize($this->_getNativePath($path, $name))) === false){
-            return PEAR::raiseError(sprintf(_("Unable to check file size of \"%s/%s\".", $path, $name)));
-        }
-
-        return $size;
-    }
-
-    /**
-     * Retrieve a file from the VFS.
-     *
-     * @param string $path  The pathname to the file.
-     * @param string $name  The filename to retrieve.
-     *
-     * @return string  The file data.
-     */
-    function read($path, $name)
-    {
-        $conn = $this->_connect();
-        if (is_a($conn, 'PEAR_Error')) {
-            return $conn;
-        }
-
-        $file = $this->_getNativePath($path, $name);
-
-        if (function_exists('file_get_contents')) {
-            $data = file_get_contents($file);
-        } else {
-            $fp = @fopen($file, 'rb');
-            if (!$fp) {
-                return PEAR::raiseError(_("Unable to open VFS file."));
-            }
-            $data = fread($fp, filesize($file));
-            fclose($fp);
-        }
-
-        return $data;
-    }
-
     /**
      * Store a file in the VFS, with the data copied from a temporary
      * file.
@@ -117,14 +76,7 @@ class VFS_sql_file extends VFS {
     {
         /* No need to check quota here as we will check it when we call
          * writeData(). */
-
-        if (function_exists('file_get_contents')) {
-            $data = file_get_contents($tmpFile);
-        } else {
-            $dataFP = @fopen($tmpFile, 'rb');
-            $data = @fread($dataFP, filesize($tmpFile));
-            fclose($dataFP);
-        }
+        $data = file_get_contents($tmpFile);
         return $this->writeData($path, $name, $data, $autocreate);
     }
 
@@ -174,25 +126,31 @@ class VFS_sql_file extends VFS {
     /**
      * Moves a file in the database and the file system.
      *
-     * @param string $path  The path to store the file in.
-     * @param string $name  The old filename.
-     * @param string $dest  The new filename.
+     * @param string $path         The path to store the file in.
+     * @param string $name         The old filename.
+     * @param string $dest         The new filename.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function move($path, $name, $dest)
+    function move($path, $name, $dest, $autocreate = false)
     {
         $orig = $this->_getNativePath($path, $name);
         if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
             return PEAR::raiseError(_("Cannot move file(s) - destination is within source."));
         }
 
-        $conn = $this->_connect();
-        if (is_a($conn, 'PEAR_Error')) {
-            return $conn;
+        if ($autocreate) {
+            $result = $this->autocreatePath($dest);
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
         }
 
         $fileCheck = $this->listFolder($dest, null, false);
+        if (is_a($fileCheck, 'PEAR_Error')) {
+            return $fileCheck;
+        }
         foreach ($fileCheck as $file) {
             if ($file['name'] == $name) {
                 return PEAR::raiseError(_("Unable to move VFS file."));
@@ -209,13 +167,14 @@ class VFS_sql_file extends VFS {
     /**
      * Copies a file through the backend.
      *
-     * @param string $path  The path to store the file in.
-     * @param string $name  The filename to use.
-     * @param string $dest  The destination of the file.
+     * @param string $path         The path to store the file in.
+     * @param string $name         The filename to use.
+     * @param string $dest         The destination of the file.
+     * @param boolean $autocreate  Automatically create directories?
      *
      * @return mixed  True on success or a PEAR_Error object on failure.
      */
-    function copy($path, $name, $dest)
+    function copy($path, $name, $dest, $autocreate = false)
     {
         $orig = $this->_getNativePath($path, $name);
         if (preg_match('|^' . preg_quote($orig) . '/?$|', $dest)) {
@@ -227,7 +186,17 @@ class VFS_sql_file extends VFS {
             return $conn;
         }
 
+        if ($autocreate) {
+            $result = $this->autocreatePath($dest);
+            if (is_a($result, 'PEAR_Error')) {
+                return $result;
+            }
+        }
+
         $fileCheck = $this->listFolder($dest, null, false);
+        if (is_a($fileCheck, 'PEAR_Error')) {
+            return $fileCheck;
+        }
         foreach ($fileCheck as $file) {
             if ($file['name'] == $name) {
                 return PEAR::raiseError(_("Unable to copy VFS file."));
@@ -607,13 +576,11 @@ class VFS_sql_file extends VFS {
     function _recursiveCopy($path, $name, $dest)
     {
         $result = $this->createFolder($dest, $name);
-
         if (is_a($result, 'PEAR_Error')) {
             return $result;
         }
 
         $file_list = $this->listFolder($this->_getSQLNativePath($path, $name));
-
         foreach ($file_list as $file) {
             $result = $this->copy($this->_getSQLNativePath($path, $name), $file['name'], $this->_getSQLNativePath($dest, $name));
 
@@ -747,7 +714,7 @@ class VFS_sql_file extends VFS {
         while (false !== ($file = readdir($dh))) {
             if ($file != '.' && $file != '..') {
                 if (is_dir($dir . '/' . $file)) {
-                    $this->_recursiveLFSDelete(empty($path) ? $name : $path . '/' . $name, $file);
+                    $this->_recursiveLFSDelete(!strlen($path) ? $name : $path . '/' . $name, $file);
                 } else {
                     @unlink($dir . '/' . $file);
                 }
@@ -767,71 +734,61 @@ class VFS_sql_file extends VFS {
      */
     function _connect()
     {
-        if ($this->_db === false) {
-            if (!is_array($this->_params)) {
-                return PEAR::raiseError(_("No configuration information specified for SQL-File VFS."));
-            }
+        if ($this->_db !== false) {
+            return true;
+        }
 
-            $required = array('phptype', 'vfsroot');
-            foreach ($required as $val) {
-                if (!isset($this->_params[$val])) {
-                    return PEAR::raiseError(sprintf(_("Required \"%s\" not specified in VFS configuration."), $val));
-                }
-            }
+        if (!is_array($this->_params)) {
+            return PEAR::raiseError(_("No configuration information specified for SQL-File VFS."));
+        }
 
-            if (!isset($this->_params['database'])) {
-                $this->_params['database'] = '';
+        $required = array('phptype', 'vfsroot');
+        foreach ($required as $val) {
+            if (!isset($this->_params[$val])) {
+                return PEAR::raiseError(sprintf(_("Required \"%s\" not specified in VFS configuration."), $val));
             }
+        }
 
-            if (!isset($this->_params['username'])) {
-                $this->_params['username'] = '';
-            }
+        if (!isset($this->_params['database'])) {
+            $this->_params['database'] = '';
+        }
 
-            if (!isset($this->_params['hostspec'])) {
-                $this->_params['hostspec'] = '';
-            }
+        if (!isset($this->_params['username'])) {
+            $this->_params['username'] = '';
+        }
 
-            if (!isset($this->_params['table'])) {
-                $this->_params['table'] = 'horde_vfs';
-            }
+        if (!isset($this->_params['hostspec'])) {
+            $this->_params['hostspec'] = '';
+        }
 
-            /* Connect to the SQL server using the supplied parameters. */
-            require_once 'DB.php';
-            $this->_db = &DB::connect($this->_params,
-                                      array('persistent' => !empty($this->_params['persistent'])));
-            if (is_a($this->_db, 'PEAR_Error')) {
-                $error = $this->_db;
-                $this->_db = false;
-                return $error;
-            }
+        if (!isset($this->_params['table'])) {
+            $this->_params['table'] = 'horde_vfs';
+        }
 
-            // Set DB portability options.
-            switch ($this->_db->phptype) {
-            case 'mssql':
-                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
-                break;
-            default:
-                $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
-            }
+        /* Connect to the SQL server using the supplied parameters. */
+        require_once 'DB.php';
+        $this->_db = &DB::connect($this->_params,
+                                  array('persistent' => !empty($this->_params['persistent']),
+                                        'ssl' => !empty($this->_params['ssl'])));
+        if (is_a($this->_db, 'PEAR_Error')) {
+            $error = $this->_db;
+            $this->_db = false;
+            return $error;
+        }
 
+        // Set DB portability options.
+        switch ($this->_db->phptype) {
+        case 'mssql':
+            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS | DB_PORTABILITY_RTRIM);
+            break;
+
+        default:
+            $this->_db->setOption('portability', DB_PORTABILITY_LOWERCASE | DB_PORTABILITY_ERRORS);
         }
 
         return true;
     }
 
-    /**
-     * Disconnect from the SQL server and clean up the connection.
-     *
-     * @access private
-     */
-    function _disconnect()
-    {
-        if ($this->_db) {
-            $this->_db->disconnect();
-            $this->_db = false;
-        }
-    }
-
     /**
      * Return a full filename on the native filesystem, from a VFS
      * path and name.
@@ -845,10 +802,10 @@ class VFS_sql_file extends VFS {
      */
     function _getNativePath($path, $name)
     {
-        if (!empty($name)) {
+        if (strlen($name)) {
             $name = '/' . $name;
         }
-        if (isset($path)) {
+        if (strlen($path)) {
             if (isset($this->_params['home']) &&
                 preg_match('|^~/?(.*)$|', $path, $matches)) {
                 $path = $this->_params['home']  . '/' . $matches[1];
@@ -873,7 +830,7 @@ class VFS_sql_file extends VFS {
      */
     function _getSQLNativePath($path, $name)
     {
-        if (empty($path)) {
+        if (!strlen($path)) {
             return $name;
         }
 
