@@ -1,4 +1,4 @@
--- $Horde: horde/scripts/sql/horde_vfs.oci8.sql,v 1.3 2004/09/21 03:34:06 chuck Exp $
+-- $Horde: horde/scripts/sql/horde_vfs.oci8.sql,v 1.3.10.1 2007/12/20 15:03:03 jan Exp $
 
 CREATE TABLE horde_vfs (
     vfs_id        NUMBER(16) NOT NULL,
@@ -14,5 +14,3 @@ CREATE TABLE horde_vfs (
 
 CREATE INDEX vfs_path_idx ON horde_vfs (vfs_path);
 CREATE INDEX vfs_name_idx ON horde_vfs (vfs_name);
-
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_vfs TO horde;
