@@ -1,7 +1,7 @@
--- $Horde: horde/scripts/sql/horde_vfs.pgsql.sql,v 1.1 2004/09/18 17:20:59 chuck Exp $
+-- $Horde: horde/scripts/sql/horde_vfs.pgsql.sql,v 1.1.10.3 2008/10/15 04:07:01 chuck Exp $
 
 CREATE TABLE horde_vfs (
-    vfs_id        BIGINT NOT NULL,
+    vfs_id        INT NOT NULL,
     vfs_type      SMALLINT NOT NULL,
     vfs_path      VARCHAR(255) NOT NULL,
     vfs_name      VARCHAR(255) NOT NULL,
@@ -14,5 +14,3 @@ CREATE TABLE horde_vfs (
 
 CREATE INDEX vfs_path_idx ON horde_vfs (vfs_path);
 CREATE INDEX vfs_name_idx ON horde_vfs (vfs_name);
-
-GRANT SELECT, INSERT, UPDATE, DELETE ON horde_vfs TO horde;
