@@ -1,8 +1,8 @@
 <?php
 /**
- * $Horde: horde/services/shares/edit.php,v 1.26.10.16 2008/04/22 16:06:53 chuck Exp $
+ * $Horde: horde/services/shares/edit.php,v 1.26.10.18 2009/09/11 13:03:23 jan Exp $
  *
- * Copyright 2002-2008 The Horde Project (http://www.horde.org/)
+ * Copyright 2002-2009 The Horde Project (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.fsf.org/copyleft/lgpl.html.
@@ -75,6 +75,8 @@ case 'editform':
         if ($old_owner !== $new_owner && !empty($new_owner)) {
             if ($old_owner != Auth::getAuth() && !Auth::isAdmin()) {
                 $notification->push(_("Only the owner or system administrator may change ownership or owner permissions for a share"), 'horde.error');
+            } elseif ($auth->hasCapability('list') && !$auth->exists($new_owner)) {
+                $notification->push(sprintf(_("The user \"%s\" does not exist."), Auth::removeHook($new_owner)), 'horde.error');
             } else {
                 $share->set('owner', $new_owner);
                 $share->save();
@@ -162,6 +164,10 @@ case 'editform':
             if (empty($user) || $user == $new_owner) {
                 continue;
             }
+            if ($auth->hasCapability('list') && !$auth->exists($user)) {
+                $notification->push(sprintf(_("The user \"%s\" does not exist."), Auth::removeHook($user)), 'horde.error');
+                continue;
+            }
 
             if (!empty($u_show[$key])) {
                 $perm->addUserPermission($user, PERMS_SHOW, false);
